#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "vk_enum_to_str.h"
#include "vk_device_info.h"
#include "vk_device.h"


bool
vk_get_physical_device_info(struct vk_device *dev) 
{
   struct vk_device_info *info = &dev->info;
   bool support_KHR_maintenance1 = false;
   bool support_KHR_maintenance2 = false;
   bool support_KHR_maintenance3 = false;
   bool support_KHR_maintenance4 = false;
   bool support_KHR_external_memory = false;
   bool support_KHR_external_memory_fd = false;
   bool support_KHR_vulkan_memory_model = false;
   bool support_KHR_external_semaphore_fd = false;
   bool support_KHR_create_renderpass2 = false;
   bool support_KHR_synchronization2 = false;
   bool support_KHR_swapchain_mutable_format = false;
   bool support_EXT_provoking_vertex = false;
   bool support_EXT_shader_viewport_index_layer = false;
   bool support_KHR_get_memory_requirements2 = false;
   bool support_EXT_post_depth_coverage = false;
   bool support_EXT_depth_clip_control = false;
   bool support_EXT_shader_subgroup_ballot = false;
   bool support_EXT_shader_subgroup_vote = false;
   bool support_EXT_shader_atomic_float = false;
   bool support_KHR_shader_atomic_int64 = false;
   bool support_KHR_8bit_storage = false;
   bool support_KHR_16bit_storage = false;
   bool support_EXT_image_2d_view_of_3d = false;
   bool support_KHR_driver_properties = false;
   bool support_EXT_memory_budget = false;
   bool support_KHR_draw_indirect_count = false;
   bool support_EXT_fragment_shader_interlock = false;
   bool support_EXT_sample_locations = false;
   bool support_EXT_conservative_rasterization = false;
   bool support_KHR_shader_draw_parameters = false;
   bool support_KHR_sampler_mirror_clamp_to_edge = false;
   bool support_KHR_buffer_device_address = false;
   bool support_EXT_descriptor_buffer = false;
   bool support_EXT_conditional_rendering = false;
   bool support_EXT_transform_feedback = false;
   bool support_EXT_index_type_uint8 = false;
   bool support_KHR_image_format_list = false;
   bool support_KHR_sampler_ycbcr_conversion = false;
   bool support_KHR_imageless_framebuffer = false;
   bool support_EXT_robustness2 = false;
   bool support_EXT_image_drm_format_modifier = false;
   bool support_EXT_vertex_attribute_divisor = false;
   bool support_EXT_calibrated_timestamps = false;
   bool support_NV_linear_color_attachment = false;
   bool support_KHR_dynamic_rendering = false;
   bool support_KHR_shader_clock = false;
   bool support_EXT_sampler_filter_minmax = false;
   bool support_EXT_custom_border_color = false;
   bool support_EXT_non_seamless_cube_map = false;
   bool support_EXT_border_color_swizzle = false;
   bool support_EXT_blend_operation_advanced = false;
   bool support_EXT_extended_dynamic_state = false;
   bool support_EXT_extended_dynamic_state2 = false;
   bool support_EXT_pipeline_creation_cache_control = false;
   bool support_EXT_shader_stencil_export = false;
#ifdef VK_KHR_PORTABILITY_SUBSET_EXTENSION_NAME
   bool support_KHR_portability_subset = false;
#endif
   bool support_NV_compute_shader_derivatives = false;
   bool support_KHR_timeline_semaphore = false;
   bool support_EXT_color_write_enable = false;
   bool support_EXT_4444_formats = false;
   bool support_EXT_scalar_block_layout = false;
   bool support_KHR_swapchain = false;
   bool support_EXT_rasterization_order_attachment_access = false;
   bool support_KHR_shader_float16_int8 = false;
   bool support_EXT_multi_draw = false;
   bool support_EXT_primitives_generated_query = false;
   bool support_KHR_pipeline_library = false;
   bool support_EXT_graphics_pipeline_library = false;
   bool support_KHR_push_descriptor = false;
   bool support_KHR_descriptor_update_template = false;
   bool support_EXT_line_rasterization = false;
   bool support_EXT_mutable_descriptor_type = false;
   bool support_EXT_vertex_input_dynamic_state = false;
   bool support_EXT_primitive_topology_list_restart = false;
   bool support_KHR_dedicated_allocation = false;
   bool support_EXT_descriptor_indexing = false;
   bool support_EXT_depth_clip_enable = false;
   bool support_EXT_shader_demote_to_helper_invocation = false;
   bool support_KHR_shader_terminate_invocation = false;
   bool support_EXT_host_image_copy = false;
   uint32_t num_extensions = 0;

   // get device memory properties
   dev->vk.GetPhysicalDeviceMemoryProperties(dev->pdev, &info->mem_props);

   // enumerate device supported extensions
   VkResult result = dev->vk.EnumerateDeviceExtensionProperties(dev->pdev, NULL, &num_extensions, NULL);
   if (result != VK_SUCCESS) {
      fprintf(stderr, "VK: vkEnumerateDeviceExtensionProperties failed (%s)\n", vk_Result_to_str(result));
   } else {
      if (num_extensions > 0) {
         VkExtensionProperties *extensions = malloc(sizeof(VkExtensionProperties) * num_extensions);
         if (!extensions) goto fail;
         result = dev->vk.EnumerateDeviceExtensionProperties(dev->pdev, NULL, &num_extensions, extensions);
         if (result != VK_SUCCESS) {
            fprintf(stderr, "VK: vkEnumerateDeviceExtensionProperties failed (%s)\n", vk_Result_to_str(result));
         }

         for (uint32_t i = 0; i < num_extensions; ++i) {
            if (!strcmp(extensions[i].extensionName, "VK_KHR_maintenance1")) {
               info->have_KHR_maintenance1 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_maintenance2")) {
               info->have_KHR_maintenance2 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_maintenance3")) {
               info->have_KHR_maintenance3 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_maintenance4")) {
               support_KHR_maintenance4 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_external_memory")) {
               info->have_KHR_external_memory = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_external_memory_fd")) {
               info->have_KHR_external_memory_fd = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_vulkan_memory_model")) {
               info->have_KHR_vulkan_memory_model = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_external_semaphore_fd")) {
               info->have_KHR_external_semaphore_fd = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_create_renderpass2")) {
               info->have_KHR_create_renderpass2 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_synchronization2")) {
               support_KHR_synchronization2 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_swapchain_mutable_format")) {
               info->have_KHR_swapchain_mutable_format = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_provoking_vertex")) {
               support_EXT_provoking_vertex = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_shader_viewport_index_layer")) {
               info->have_EXT_shader_viewport_index_layer = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_get_memory_requirements2")) {
               info->have_KHR_get_memory_requirements2 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_post_depth_coverage")) {
               info->have_EXT_post_depth_coverage = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_depth_clip_control")) {
               support_EXT_depth_clip_control = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_shader_subgroup_ballot")) {
               info->have_EXT_shader_subgroup_ballot = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_shader_subgroup_vote")) {
               info->have_EXT_shader_subgroup_vote = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_shader_atomic_float")) {
               support_EXT_shader_atomic_float = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_shader_atomic_int64")) {
               support_KHR_shader_atomic_int64 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_8bit_storage")) {
               support_KHR_8bit_storage = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_16bit_storage")) {
               support_KHR_16bit_storage = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_image_2d_view_of_3d")) {
               support_EXT_image_2d_view_of_3d = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_driver_properties")) {
               support_KHR_driver_properties = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_memory_budget")) {
               info->have_EXT_memory_budget = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_draw_indirect_count")) {
               info->have_KHR_draw_indirect_count = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_fragment_shader_interlock")) {
               support_EXT_fragment_shader_interlock = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_sample_locations")) {
               support_EXT_sample_locations = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_conservative_rasterization")) {
               support_EXT_conservative_rasterization = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_shader_draw_parameters")) {
               info->have_KHR_shader_draw_parameters = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_sampler_mirror_clamp_to_edge")) {
               info->have_KHR_sampler_mirror_clamp_to_edge = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_buffer_device_address")) {
               support_KHR_buffer_device_address = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_descriptor_buffer")) {
               support_EXT_descriptor_buffer = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_conditional_rendering")) {
               support_EXT_conditional_rendering = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_transform_feedback")) {
               support_EXT_transform_feedback = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_index_type_uint8")) {
               support_EXT_index_type_uint8 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_image_format_list")) {
               info->have_KHR_image_format_list = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_sampler_ycbcr_conversion")) {
               info->have_KHR_sampler_ycbcr_conversion = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_imageless_framebuffer")) {
               support_KHR_imageless_framebuffer = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_robustness2")) {
               support_EXT_robustness2 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_image_drm_format_modifier")) {
               info->have_EXT_image_drm_format_modifier = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_vertex_attribute_divisor")) {
               support_EXT_vertex_attribute_divisor = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_calibrated_timestamps")) {
               info->have_EXT_calibrated_timestamps = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_NV_linear_color_attachment")) {
               support_NV_linear_color_attachment = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_dynamic_rendering")) {
               support_KHR_dynamic_rendering = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_shader_clock")) {
               support_KHR_shader_clock = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_sampler_filter_minmax")) {
               support_EXT_sampler_filter_minmax = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_custom_border_color")) {
               support_EXT_custom_border_color = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_non_seamless_cube_map")) {
               support_EXT_non_seamless_cube_map = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_border_color_swizzle")) {
               support_EXT_border_color_swizzle = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_blend_operation_advanced")) {
               support_EXT_blend_operation_advanced = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_extended_dynamic_state")) {
               support_EXT_extended_dynamic_state = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_extended_dynamic_state2")) {
               support_EXT_extended_dynamic_state2 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_pipeline_creation_cache_control")) {
               support_EXT_pipeline_creation_cache_control = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_shader_stencil_export")) {
               info->have_EXT_shader_stencil_export = true;
            }
         #ifdef VK_KHR_PORTABILITY_SUBSET_EXTENSION_NAME
   if (!strcmp(extensions[i].extensionName, "VK_KHR_portability_subset")) {
               support_KHR_portability_subset = true;
            }
#endif
            if (!strcmp(extensions[i].extensionName, "VK_NV_compute_shader_derivatives")) {
               support_NV_compute_shader_derivatives = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_timeline_semaphore")) {
               support_KHR_timeline_semaphore = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_color_write_enable")) {
               support_EXT_color_write_enable = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_4444_formats")) {
               support_EXT_4444_formats = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_scalar_block_layout")) {
               support_EXT_scalar_block_layout = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_swapchain")) {
               info->have_KHR_swapchain = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_rasterization_order_attachment_access")) {
               support_EXT_rasterization_order_attachment_access = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_shader_float16_int8")) {
               support_KHR_shader_float16_int8 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_multi_draw")) {
               support_EXT_multi_draw = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_primitives_generated_query")) {
               support_EXT_primitives_generated_query = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_pipeline_library")) {
               info->have_KHR_pipeline_library = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_graphics_pipeline_library")) {
               support_EXT_graphics_pipeline_library = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_push_descriptor")) {
               support_KHR_push_descriptor = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_descriptor_update_template")) {
               info->have_KHR_descriptor_update_template = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_line_rasterization")) {
               support_EXT_line_rasterization = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_mutable_descriptor_type")) {
               support_EXT_mutable_descriptor_type = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_vertex_input_dynamic_state")) {
               support_EXT_vertex_input_dynamic_state = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_primitive_topology_list_restart")) {
               support_EXT_primitive_topology_list_restart = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_dedicated_allocation")) {
               info->have_KHR_dedicated_allocation = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_descriptor_indexing")) {
               support_EXT_descriptor_indexing = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_depth_clip_enable")) {
               support_EXT_depth_clip_enable = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_shader_demote_to_helper_invocation")) {
               support_EXT_shader_demote_to_helper_invocation = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_shader_terminate_invocation")) {
               support_KHR_shader_terminate_invocation = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_host_image_copy")) {
               info->have_EXT_host_image_copy = true;
            }
         }

         free(extensions);
      }
   }

   // get device features
   if (dev->vk.GetPhysicalDeviceFeatures2) {
      // check for device extension features
      info->feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2;

      if (VK_MAKE_VERSION(1,2,0) <= dev->vk_version) {
         /* VkPhysicalDeviceVulkan11Features was added in 1.2, not 1.1 as one would think */
         info->feats11.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_1_FEATURES;
         info->feats11.pNext = info->feats.pNext;
         info->feats.pNext = &info->feats11;
         info->have_vulkan11 = true;
      }
      if (VK_MAKE_VERSION(1,2,0) <= dev->vk_version) {
         info->feats12.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_FEATURES;
         info->feats12.pNext = info->feats.pNext;
         info->feats.pNext = &info->feats12;
         info->have_vulkan12 = true;
      }

   if (support_KHR_maintenance4) {
         info->maint4_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_4_FEATURES_KHR;
         info->maint4_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->maint4_feats;
      }
   if (support_KHR_synchronization2) {
         info->sync2_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SYNCHRONIZATION_2_FEATURES_KHR;
         info->sync2_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->sync2_feats;
      }
   if (support_EXT_provoking_vertex) {
         info->pv_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROVOKING_VERTEX_FEATURES_EXT;
         info->pv_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->pv_feats;
      }
   if (support_EXT_depth_clip_control) {
         info->clip_control_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_CLIP_CONTROL_FEATURES_EXT;
         info->clip_control_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->clip_control_feats;
      }
   if (support_EXT_shader_atomic_float) {
         info->atomic_float_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_ATOMIC_FLOAT_FEATURES_EXT;
         info->atomic_float_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->atomic_float_feats;
      }
   if (support_KHR_shader_atomic_int64 && !info->have_vulkan12) {
         info->atomic_int_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_ATOMIC_INT64_FEATURES_KHR;
         info->atomic_int_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->atomic_int_feats;
      }
   if (support_KHR_8bit_storage && !info->have_vulkan12) {
         info->storage_8bit_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_8BIT_STORAGE_FEATURES_KHR;
         info->storage_8bit_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->storage_8bit_feats;
      }
   if (support_KHR_16bit_storage && !info->have_vulkan11) {
         info->storage_16bit_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_16BIT_STORAGE_FEATURES_KHR;
         info->storage_16bit_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->storage_16bit_feats;
      }
   if (support_EXT_image_2d_view_of_3d) {
         info->view2d_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_2D_VIEW_OF_3D_FEATURES_EXT;
         info->view2d_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->view2d_feats;
      }
   if (support_EXT_fragment_shader_interlock) {
         info->interlock_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADER_INTERLOCK_FEATURES_EXT;
         info->interlock_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->interlock_feats;
      }
   if (support_KHR_buffer_device_address && !info->have_vulkan12) {
         info->bda_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BUFFER_DEVICE_ADDRESS_FEATURES_KHR;
         info->bda_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->bda_feats;
      }
   if (support_EXT_descriptor_buffer) {
         info->db_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_BUFFER_FEATURES_EXT;
         info->db_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->db_feats;
      }
   if (support_EXT_conditional_rendering) {
         info->cond_render_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CONDITIONAL_RENDERING_FEATURES_EXT;
         info->cond_render_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->cond_render_feats;
      }
   if (support_EXT_transform_feedback) {
         info->tf_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_FEATURES_EXT;
         info->tf_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->tf_feats;
      }
   if (support_EXT_index_type_uint8) {
         info->index_uint8_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_INDEX_TYPE_UINT8_FEATURES_EXT;
         info->index_uint8_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->index_uint8_feats;
      }
   if (support_KHR_imageless_framebuffer && !info->have_vulkan12) {
         info->imgless_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGELESS_FRAMEBUFFER_FEATURES_KHR;
         info->imgless_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->imgless_feats;
      }
   if (support_EXT_robustness2) {
         info->rb2_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ROBUSTNESS_2_FEATURES_EXT;
         info->rb2_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->rb2_feats;
      }
   if (support_EXT_vertex_attribute_divisor) {
         info->vdiv_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_FEATURES_EXT;
         info->vdiv_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->vdiv_feats;
      }
   if (support_NV_linear_color_attachment) {
         info->linear_color_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINEAR_COLOR_ATTACHMENT_FEATURES_NV;
         info->linear_color_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->linear_color_feats;
      }
   if (support_KHR_dynamic_rendering) {
         info->dynamic_render_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DYNAMIC_RENDERING_FEATURES_KHR;
         info->dynamic_render_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->dynamic_render_feats;
      }
   if (support_KHR_shader_clock) {
         info->shader_clock_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_CLOCK_FEATURES_KHR;
         info->shader_clock_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->shader_clock_feats;
      }
   if (support_EXT_custom_border_color) {
         info->border_color_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUSTOM_BORDER_COLOR_FEATURES_EXT;
         info->border_color_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->border_color_feats;
      }
   if (support_EXT_non_seamless_cube_map) {
         info->nonseamless_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_NON_SEAMLESS_CUBE_MAP_FEATURES_EXT;
         info->nonseamless_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->nonseamless_feats;
      }
   if (support_EXT_border_color_swizzle) {
         info->border_swizzle_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BORDER_COLOR_SWIZZLE_FEATURES_EXT;
         info->border_swizzle_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->border_swizzle_feats;
      }
   if (support_EXT_extended_dynamic_state) {
         info->dynamic_state_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_FEATURES_EXT;
         info->dynamic_state_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->dynamic_state_feats;
      }
   if (support_EXT_extended_dynamic_state2) {
         info->dynamic_state2_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_2_FEATURES_EXT;
         info->dynamic_state2_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->dynamic_state2_feats;
      }
   if (support_EXT_pipeline_creation_cache_control) {
         info->pipeline_cache_control_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_CREATION_CACHE_CONTROL_FEATURES_EXT;
         info->pipeline_cache_control_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->pipeline_cache_control_feats;
      }
#ifdef VK_KHR_PORTABILITY_SUBSET_EXTENSION_NAME
   if (support_KHR_portability_subset) {
         info->portability_subset_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PORTABILITY_SUBSET_FEATURES_KHR;
         info->portability_subset_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->portability_subset_feats;
      }
#endif
   if (support_NV_compute_shader_derivatives) {
         info->shader_derivs_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COMPUTE_SHADER_DERIVATIVES_FEATURES_NV;
         info->shader_derivs_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->shader_derivs_feats;
      }
   if (support_KHR_timeline_semaphore && !info->have_vulkan12) {
         info->timeline_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TIMELINE_SEMAPHORE_FEATURES_KHR;
         info->timeline_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->timeline_feats;
      }
   if (support_EXT_color_write_enable) {
         info->cwrite_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COLOR_WRITE_ENABLE_FEATURES_EXT;
         info->cwrite_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->cwrite_feats;
      }
   if (support_EXT_4444_formats) {
         info->format_4444_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_4444_FORMATS_FEATURES_EXT;
         info->format_4444_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->format_4444_feats;
      }
   if (support_EXT_scalar_block_layout && !info->have_vulkan12) {
         info->scalar_block_layout_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SCALAR_BLOCK_LAYOUT_FEATURES_EXT;
         info->scalar_block_layout_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->scalar_block_layout_feats;
      }
   if (support_EXT_rasterization_order_attachment_access) {
         info->rast_order_access_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RASTERIZATION_ORDER_ATTACHMENT_ACCESS_FEATURES_EXT;
         info->rast_order_access_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->rast_order_access_feats;
      }
   if (support_KHR_shader_float16_int8 && !info->have_vulkan12) {
         info->shader_float16_int8_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_FLOAT16_INT8_FEATURES_KHR;
         info->shader_float16_int8_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->shader_float16_int8_feats;
      }
   if (support_EXT_multi_draw) {
         info->multidraw_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTI_DRAW_FEATURES_EXT;
         info->multidraw_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->multidraw_feats;
      }
   if (support_EXT_primitives_generated_query) {
         info->primgen_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRIMITIVES_GENERATED_QUERY_FEATURES_EXT;
         info->primgen_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->primgen_feats;
      }
   if (support_EXT_graphics_pipeline_library) {
         info->gpl_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GRAPHICS_PIPELINE_LIBRARY_FEATURES_EXT;
         info->gpl_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->gpl_feats;
      }
   if (support_EXT_line_rasterization) {
         info->line_rast_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINE_RASTERIZATION_FEATURES_EXT;
         info->line_rast_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->line_rast_feats;
      }
   if (support_EXT_mutable_descriptor_type) {
         info->mutable_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MUTABLE_DESCRIPTOR_TYPE_FEATURES_EXT;
         info->mutable_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->mutable_feats;
      }
   if (support_EXT_vertex_input_dynamic_state) {
         info->vertex_input_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_INPUT_DYNAMIC_STATE_FEATURES_EXT;
         info->vertex_input_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->vertex_input_feats;
      }
   if (support_EXT_primitive_topology_list_restart) {
         info->list_restart_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRIMITIVE_TOPOLOGY_LIST_RESTART_FEATURES_EXT;
         info->list_restart_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->list_restart_feats;
      }
   if (support_EXT_descriptor_indexing && !info->have_vulkan12) {
         info->desc_indexing_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_INDEXING_FEATURES_EXT;
         info->desc_indexing_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->desc_indexing_feats;
      }
   if (support_EXT_depth_clip_enable) {
         info->depth_clip_enable_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_CLIP_ENABLE_FEATURES_EXT;
         info->depth_clip_enable_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->depth_clip_enable_feats;
      }
   if (support_EXT_shader_demote_to_helper_invocation) {
         info->demote_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_DEMOTE_TO_HELPER_INVOCATION_FEATURES_EXT;
         info->demote_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->demote_feats;
      }
   if (support_KHR_shader_terminate_invocation) {
         info->terminate_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_TERMINATE_INVOCATION_FEATURES_KHR;
         info->terminate_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->terminate_feats;
      }

      dev->vk.GetPhysicalDeviceFeatures2(dev->pdev, &info->feats);
   } else {
      dev->vk.GetPhysicalDeviceFeatures(dev->pdev, &info->feats.features);
   }

   // check for device properties
   if (dev->vk.GetPhysicalDeviceProperties2) {
      VkPhysicalDeviceProperties2 props = {0};
      props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROPERTIES_2;

      if (VK_MAKE_VERSION(1,2,0) <= dev->vk_version) {
         /* VkPhysicalDeviceVulkan11Properties was added in 1.2, not 1.1 as one would think */
         info->props11.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_1_PROPERTIES;
         info->props11.pNext = props.pNext;
         props.pNext = &info->props11;
      }
      if (VK_MAKE_VERSION(1,2,0) <= dev->vk_version) {
         info->props12.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_PROPERTIES;
         info->props12.pNext = props.pNext;
         props.pNext = &info->props12;
      }

   if (support_EXT_provoking_vertex) {
         info->pv_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROVOKING_VERTEX_PROPERTIES_EXT;
         info->pv_props.pNext = props.pNext;
         props.pNext = &info->pv_props;
      }
   if (support_KHR_driver_properties && !info->have_vulkan12) {
         info->driver_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DRIVER_PROPERTIES_KHR;
         info->driver_props.pNext = props.pNext;
         props.pNext = &info->driver_props;
      }
   if (support_EXT_sample_locations) {
         info->sample_locations_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SAMPLE_LOCATIONS_PROPERTIES_EXT;
         info->sample_locations_props.pNext = props.pNext;
         props.pNext = &info->sample_locations_props;
      }
   if (support_EXT_conservative_rasterization) {
         info->cons_raster_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CONSERVATIVE_RASTERIZATION_PROPERTIES_EXT;
         info->cons_raster_props.pNext = props.pNext;
         props.pNext = &info->cons_raster_props;
      }
   if (support_EXT_descriptor_buffer) {
         info->db_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_BUFFER_PROPERTIES_EXT;
         info->db_props.pNext = props.pNext;
         props.pNext = &info->db_props;
      }
   if (support_EXT_transform_feedback) {
         info->tf_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_PROPERTIES_EXT;
         info->tf_props.pNext = props.pNext;
         props.pNext = &info->tf_props;
      }
   if (support_EXT_robustness2) {
         info->rb2_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ROBUSTNESS_2_PROPERTIES_EXT;
         info->rb2_props.pNext = props.pNext;
         props.pNext = &info->rb2_props;
      }
   if (support_EXT_vertex_attribute_divisor) {
         info->vdiv_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_PROPERTIES_EXT;
         info->vdiv_props.pNext = props.pNext;
         props.pNext = &info->vdiv_props;
      }
   if (support_EXT_sampler_filter_minmax && !info->have_vulkan12) {
         info->reduction_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SAMPLER_FILTER_MINMAX_PROPERTIES_EXT;
         info->reduction_props.pNext = props.pNext;
         props.pNext = &info->reduction_props;
      }
   if (support_EXT_custom_border_color) {
         info->border_color_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUSTOM_BORDER_COLOR_PROPERTIES_EXT;
         info->border_color_props.pNext = props.pNext;
         props.pNext = &info->border_color_props;
      }
   if (support_EXT_blend_operation_advanced) {
         info->blend_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BLEND_OPERATION_ADVANCED_PROPERTIES_EXT;
         info->blend_props.pNext = props.pNext;
         props.pNext = &info->blend_props;
      }
   if (support_EXT_multi_draw) {
         info->multidraw_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTI_DRAW_PROPERTIES_EXT;
         info->multidraw_props.pNext = props.pNext;
         props.pNext = &info->multidraw_props;
      }
   if (support_EXT_graphics_pipeline_library) {
         info->gpl_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GRAPHICS_PIPELINE_LIBRARY_PROPERTIES_EXT;
         info->gpl_props.pNext = props.pNext;
         props.pNext = &info->gpl_props;
      }
   if (support_KHR_push_descriptor) {
         info->push_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PUSH_DESCRIPTOR_PROPERTIES_KHR;
         info->push_props.pNext = props.pNext;
         props.pNext = &info->push_props;
      }
   if (support_EXT_line_rasterization) {
         info->line_rast_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINE_RASTERIZATION_PROPERTIES_EXT;
         info->line_rast_props.pNext = props.pNext;
         props.pNext = &info->line_rast_props;
      }
   if (support_EXT_descriptor_indexing && !info->have_vulkan12) {
         info->desc_indexing_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_INDEXING_PROPERTIES_EXT;
         info->desc_indexing_props.pNext = props.pNext;
         props.pNext = &info->desc_indexing_props;
      }

      if (dev->vk_version < VK_MAKE_VERSION(1,2,0) && dev->instance_info.have_KHR_external_memory_capabilities) {
         info->deviceid_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ID_PROPERTIES;
         info->deviceid_props.pNext = props.pNext;
         props.pNext = &info->deviceid_props;
      }

      if (dev->vk_version >= VK_MAKE_VERSION(1,1,0)) {
         info->subgroup.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBGROUP_PROPERTIES;
         info->subgroup.pNext = props.pNext;
         props.pNext = &info->subgroup;
      }

      // note: setting up local VkPhysicalDeviceProperties2.
      dev->vk.GetPhysicalDeviceProperties2(dev->pdev, &props);
   }

   /* We re-apply the fields from VkPhysicalDeviceVulkanXYFeatures struct
    * onto their respective fields in the VkPhysicalDeviceExtensionNameFeatures
    * struct if the former is provided by the VK implementation.
    *
    * As for why this is done: the spec mentions that once an extension is
    * promoted to core and its feature fields are added in VulkanXYFeatures,
    * including both ExtensionNameFeatures and VulkanXYFeatures at the same
    * time is prohibited when using vkGetPhysicalDeviceFeatures2.
    */
   if (info->have_vulkan12) {
      info->atomic_int_feats.shaderBufferInt64Atomics = info->feats12.shaderBufferInt64Atomics;
      info->atomic_int_feats.shaderSharedInt64Atomics = info->feats12.shaderSharedInt64Atomics;
   }
   if (info->have_vulkan12) {
      info->storage_8bit_feats.storageBuffer8BitAccess = info->feats12.storageBuffer8BitAccess;
      info->storage_8bit_feats.uniformAndStorageBuffer8BitAccess = info->feats12.uniformAndStorageBuffer8BitAccess;
      info->storage_8bit_feats.storagePushConstant8 = info->feats12.storagePushConstant8;
   }
   if (info->have_vulkan11) {
      info->storage_16bit_feats.storageBuffer16BitAccess = info->feats11.storageBuffer16BitAccess;
      info->storage_16bit_feats.uniformAndStorageBuffer16BitAccess = info->feats11.uniformAndStorageBuffer16BitAccess;
      info->storage_16bit_feats.storagePushConstant16 = info->feats11.storagePushConstant16;
      info->storage_16bit_feats.storageInputOutput16 = info->feats11.storageInputOutput16;
   }
   if (info->have_vulkan12) {
      info->bda_feats.bufferDeviceAddress = info->feats12.bufferDeviceAddress;
      info->bda_feats.bufferDeviceAddressCaptureReplay = info->feats12.bufferDeviceAddressCaptureReplay;
      info->bda_feats.bufferDeviceAddressMultiDevice = info->feats12.bufferDeviceAddressMultiDevice;
   }
   if (info->have_vulkan12) {
      info->imgless_feats.imagelessFramebuffer = info->feats12.imagelessFramebuffer;
   }
   if (info->have_vulkan12) {
      info->timeline_feats.timelineSemaphore = info->feats12.timelineSemaphore;
   }
   if (info->have_vulkan12) {
      info->scalar_block_layout_feats.scalarBlockLayout = info->feats12.scalarBlockLayout;
   }
   if (info->have_vulkan12) {
      info->shader_float16_int8_feats.shaderFloat16 = info->feats12.shaderFloat16;
      info->shader_float16_int8_feats.shaderInt8 = info->feats12.shaderInt8;
   }
   if (info->have_vulkan12) {
      info->desc_indexing_feats.shaderInputAttachmentArrayDynamicIndexing = info->feats12.shaderInputAttachmentArrayDynamicIndexing;
      info->desc_indexing_feats.shaderUniformTexelBufferArrayDynamicIndexing = info->feats12.shaderUniformTexelBufferArrayDynamicIndexing;
      info->desc_indexing_feats.shaderStorageTexelBufferArrayDynamicIndexing = info->feats12.shaderStorageTexelBufferArrayDynamicIndexing;
      info->desc_indexing_feats.shaderUniformBufferArrayNonUniformIndexing = info->feats12.shaderUniformBufferArrayNonUniformIndexing;
      info->desc_indexing_feats.shaderSampledImageArrayNonUniformIndexing = info->feats12.shaderSampledImageArrayNonUniformIndexing;
      info->desc_indexing_feats.shaderStorageBufferArrayNonUniformIndexing = info->feats12.shaderStorageBufferArrayNonUniformIndexing;
      info->desc_indexing_feats.shaderStorageImageArrayNonUniformIndexing = info->feats12.shaderStorageImageArrayNonUniformIndexing;
      info->desc_indexing_feats.shaderInputAttachmentArrayNonUniformIndexing = info->feats12.shaderInputAttachmentArrayNonUniformIndexing;
      info->desc_indexing_feats.shaderUniformTexelBufferArrayNonUniformIndexing = info->feats12.shaderUniformTexelBufferArrayNonUniformIndexing;
      info->desc_indexing_feats.shaderStorageTexelBufferArrayNonUniformIndexing = info->feats12.shaderStorageTexelBufferArrayNonUniformIndexing;
      info->desc_indexing_feats.descriptorBindingUniformBufferUpdateAfterBind = info->feats12.descriptorBindingUniformBufferUpdateAfterBind;
      info->desc_indexing_feats.descriptorBindingSampledImageUpdateAfterBind = info->feats12.descriptorBindingSampledImageUpdateAfterBind;
      info->desc_indexing_feats.descriptorBindingStorageImageUpdateAfterBind = info->feats12.descriptorBindingStorageImageUpdateAfterBind;
      info->desc_indexing_feats.descriptorBindingStorageBufferUpdateAfterBind = info->feats12.descriptorBindingStorageBufferUpdateAfterBind;
      info->desc_indexing_feats.descriptorBindingUniformTexelBufferUpdateAfterBind = info->feats12.descriptorBindingUniformTexelBufferUpdateAfterBind;
      info->desc_indexing_feats.descriptorBindingStorageTexelBufferUpdateAfterBind = info->feats12.descriptorBindingStorageTexelBufferUpdateAfterBind;
      info->desc_indexing_feats.descriptorBindingUpdateUnusedWhilePending = info->feats12.descriptorBindingUpdateUnusedWhilePending;
      info->desc_indexing_feats.descriptorBindingPartiallyBound = info->feats12.descriptorBindingPartiallyBound;
      info->desc_indexing_feats.descriptorBindingVariableDescriptorCount = info->feats12.descriptorBindingVariableDescriptorCount;
      info->desc_indexing_feats.runtimeDescriptorArray = info->feats12.runtimeDescriptorArray;
   }

   /* See above, but for VulkanXYProperties.
    * Unlike VulkanXYFeatures with all the booleans, VulkanXYProperties can
    * contain different types of data, including arrays. The C language hates us
    * when we assign an array to another array, therefore we use an memcpy here.
    */
   if (info->have_vulkan12) {
      memcpy(&info->driver_props.driverID,
             &info->props12.driverID,
             sizeof(info->driver_props.driverID));
      memcpy(&info->driver_props.driverName,
             &info->props12.driverName,
             sizeof(info->driver_props.driverName));
      memcpy(&info->driver_props.driverInfo,
             &info->props12.driverInfo,
             sizeof(info->driver_props.driverInfo));
      memcpy(&info->driver_props.conformanceVersion,
             &info->props12.conformanceVersion,
             sizeof(info->driver_props.conformanceVersion));
   }
   if (info->have_vulkan12) {
      memcpy(&info->reduction_props.filterMinmaxSingleComponentFormats,
             &info->props12.filterMinmaxSingleComponentFormats,
             sizeof(info->reduction_props.filterMinmaxSingleComponentFormats));
      memcpy(&info->reduction_props.filterMinmaxImageComponentMapping,
             &info->props12.filterMinmaxImageComponentMapping,
             sizeof(info->reduction_props.filterMinmaxImageComponentMapping));
   }
   if (info->have_vulkan12) {
      memcpy(&info->desc_indexing_props.maxUpdateAfterBindDescriptorsInAllPools,
             &info->props12.maxUpdateAfterBindDescriptorsInAllPools,
             sizeof(info->desc_indexing_props.maxUpdateAfterBindDescriptorsInAllPools));
      memcpy(&info->desc_indexing_props.shaderUniformBufferArrayNonUniformIndexingNative,
             &info->props12.shaderUniformBufferArrayNonUniformIndexingNative,
             sizeof(info->desc_indexing_props.shaderUniformBufferArrayNonUniformIndexingNative));
      memcpy(&info->desc_indexing_props.shaderSampledImageArrayNonUniformIndexingNative,
             &info->props12.shaderSampledImageArrayNonUniformIndexingNative,
             sizeof(info->desc_indexing_props.shaderSampledImageArrayNonUniformIndexingNative));
      memcpy(&info->desc_indexing_props.shaderStorageBufferArrayNonUniformIndexingNative,
             &info->props12.shaderStorageBufferArrayNonUniformIndexingNative,
             sizeof(info->desc_indexing_props.shaderStorageBufferArrayNonUniformIndexingNative));
      memcpy(&info->desc_indexing_props.shaderStorageImageArrayNonUniformIndexingNative,
             &info->props12.shaderStorageImageArrayNonUniformIndexingNative,
             sizeof(info->desc_indexing_props.shaderStorageImageArrayNonUniformIndexingNative));
      memcpy(&info->desc_indexing_props.shaderInputAttachmentArrayNonUniformIndexingNative,
             &info->props12.shaderInputAttachmentArrayNonUniformIndexingNative,
             sizeof(info->desc_indexing_props.shaderInputAttachmentArrayNonUniformIndexingNative));
      memcpy(&info->desc_indexing_props.robustBufferAccessUpdateAfterBind,
             &info->props12.robustBufferAccessUpdateAfterBind,
             sizeof(info->desc_indexing_props.robustBufferAccessUpdateAfterBind));
      memcpy(&info->desc_indexing_props.quadDivergentImplicitLod,
             &info->props12.quadDivergentImplicitLod,
             sizeof(info->desc_indexing_props.quadDivergentImplicitLod));
      memcpy(&info->desc_indexing_props.maxPerStageDescriptorUpdateAfterBindSamplers,
             &info->props12.maxPerStageDescriptorUpdateAfterBindSamplers,
             sizeof(info->desc_indexing_props.maxPerStageDescriptorUpdateAfterBindSamplers));
      memcpy(&info->desc_indexing_props.maxPerStageDescriptorUpdateAfterBindUniformBuffers,
             &info->props12.maxPerStageDescriptorUpdateAfterBindUniformBuffers,
             sizeof(info->desc_indexing_props.maxPerStageDescriptorUpdateAfterBindUniformBuffers));
      memcpy(&info->desc_indexing_props.maxPerStageDescriptorUpdateAfterBindStorageBuffers,
             &info->props12.maxPerStageDescriptorUpdateAfterBindStorageBuffers,
             sizeof(info->desc_indexing_props.maxPerStageDescriptorUpdateAfterBindStorageBuffers));
      memcpy(&info->desc_indexing_props.maxPerStageDescriptorUpdateAfterBindSampledImages,
             &info->props12.maxPerStageDescriptorUpdateAfterBindSampledImages,
             sizeof(info->desc_indexing_props.maxPerStageDescriptorUpdateAfterBindSampledImages));
      memcpy(&info->desc_indexing_props.maxPerStageDescriptorUpdateAfterBindStorageImages,
             &info->props12.maxPerStageDescriptorUpdateAfterBindStorageImages,
             sizeof(info->desc_indexing_props.maxPerStageDescriptorUpdateAfterBindStorageImages));
      memcpy(&info->desc_indexing_props.maxPerStageDescriptorUpdateAfterBindInputAttachments,
             &info->props12.maxPerStageDescriptorUpdateAfterBindInputAttachments,
             sizeof(info->desc_indexing_props.maxPerStageDescriptorUpdateAfterBindInputAttachments));
      memcpy(&info->desc_indexing_props.maxPerStageUpdateAfterBindResources,
             &info->props12.maxPerStageUpdateAfterBindResources,
             sizeof(info->desc_indexing_props.maxPerStageUpdateAfterBindResources));
      memcpy(&info->desc_indexing_props.maxDescriptorSetUpdateAfterBindSamplers,
             &info->props12.maxDescriptorSetUpdateAfterBindSamplers,
             sizeof(info->desc_indexing_props.maxDescriptorSetUpdateAfterBindSamplers));
      memcpy(&info->desc_indexing_props.maxDescriptorSetUpdateAfterBindUniformBuffers,
             &info->props12.maxDescriptorSetUpdateAfterBindUniformBuffers,
             sizeof(info->desc_indexing_props.maxDescriptorSetUpdateAfterBindUniformBuffers));
      memcpy(&info->desc_indexing_props.maxDescriptorSetUpdateAfterBindUniformBuffersDynamic,
             &info->props12.maxDescriptorSetUpdateAfterBindUniformBuffersDynamic,
             sizeof(info->desc_indexing_props.maxDescriptorSetUpdateAfterBindUniformBuffersDynamic));
      memcpy(&info->desc_indexing_props.maxDescriptorSetUpdateAfterBindStorageBuffers,
             &info->props12.maxDescriptorSetUpdateAfterBindStorageBuffers,
             sizeof(info->desc_indexing_props.maxDescriptorSetUpdateAfterBindStorageBuffers));
      memcpy(&info->desc_indexing_props.maxDescriptorSetUpdateAfterBindStorageBuffersDynamic,
             &info->props12.maxDescriptorSetUpdateAfterBindStorageBuffersDynamic,
             sizeof(info->desc_indexing_props.maxDescriptorSetUpdateAfterBindStorageBuffersDynamic));
      memcpy(&info->desc_indexing_props.maxDescriptorSetUpdateAfterBindSampledImages,
             &info->props12.maxDescriptorSetUpdateAfterBindSampledImages,
             sizeof(info->desc_indexing_props.maxDescriptorSetUpdateAfterBindSampledImages));
      memcpy(&info->desc_indexing_props.maxDescriptorSetUpdateAfterBindStorageImages,
             &info->props12.maxDescriptorSetUpdateAfterBindStorageImages,
             sizeof(info->desc_indexing_props.maxDescriptorSetUpdateAfterBindStorageImages));
      memcpy(&info->desc_indexing_props.maxDescriptorSetUpdateAfterBindInputAttachments,
             &info->props12.maxDescriptorSetUpdateAfterBindInputAttachments,
             sizeof(info->desc_indexing_props.maxDescriptorSetUpdateAfterBindInputAttachments));
   }

   // enable the extensions if they match the conditions given by ext.enable_conds 
   if (dev->vk.GetPhysicalDeviceProperties2) {
   info->have_KHR_maintenance1 |= support_KHR_maintenance1
         ;
   info->have_KHR_maintenance2 |= support_KHR_maintenance2
         ;
   info->have_KHR_maintenance3 |= support_KHR_maintenance3
         ;
   info->have_KHR_maintenance4 |= support_KHR_maintenance4
         ;
   info->have_KHR_external_memory |= support_KHR_external_memory
         ;
   info->have_KHR_external_memory_fd |= support_KHR_external_memory_fd
         ;
   info->have_KHR_vulkan_memory_model |= support_KHR_vulkan_memory_model
         ;
   info->have_KHR_external_semaphore_fd |= support_KHR_external_semaphore_fd
         ;
   info->have_KHR_create_renderpass2 |= support_KHR_create_renderpass2
         ;
   info->have_KHR_synchronization2 |= support_KHR_synchronization2
         ;
   info->have_KHR_swapchain_mutable_format |= support_KHR_swapchain_mutable_format
         ;
   info->have_EXT_provoking_vertex |= support_EXT_provoking_vertex
         && (info->pv_feats.provokingVertexLast);
   info->have_EXT_shader_viewport_index_layer |= support_EXT_shader_viewport_index_layer
         ;
   info->have_KHR_get_memory_requirements2 |= support_KHR_get_memory_requirements2
         ;
   info->have_EXT_post_depth_coverage |= support_EXT_post_depth_coverage
         ;
   info->have_EXT_depth_clip_control |= support_EXT_depth_clip_control
         ;
   info->have_EXT_shader_subgroup_ballot |= support_EXT_shader_subgroup_ballot
         ;
   info->have_EXT_shader_subgroup_vote |= support_EXT_shader_subgroup_vote
         ;
   info->have_EXT_shader_atomic_float |= support_EXT_shader_atomic_float
         ;
   info->have_KHR_shader_atomic_int64 |= support_KHR_shader_atomic_int64
         ;
   info->have_KHR_8bit_storage |= support_KHR_8bit_storage
         && (info->storage_8bit_feats.storageBuffer8BitAccess);
   info->have_KHR_16bit_storage |= support_KHR_16bit_storage
         && (info->storage_16bit_feats.storageBuffer16BitAccess);
   info->have_EXT_image_2d_view_of_3d |= support_EXT_image_2d_view_of_3d
         ;
   info->have_KHR_driver_properties |= support_KHR_driver_properties
         ;
   info->have_EXT_memory_budget |= support_EXT_memory_budget
         ;
   info->have_KHR_draw_indirect_count |= support_KHR_draw_indirect_count
         ;
   info->have_EXT_fragment_shader_interlock |= support_EXT_fragment_shader_interlock
         && (info->interlock_feats.fragmentShaderSampleInterlock)
&& (info->interlock_feats.fragmentShaderPixelInterlock);
   info->have_EXT_sample_locations |= support_EXT_sample_locations
         ;
   info->have_EXT_conservative_rasterization |= support_EXT_conservative_rasterization
         && (info->cons_raster_props.fullyCoveredFragmentShaderInputVariable);
   info->have_KHR_shader_draw_parameters |= support_KHR_shader_draw_parameters
         ;
   info->have_KHR_sampler_mirror_clamp_to_edge |= support_KHR_sampler_mirror_clamp_to_edge
         ;
   info->have_KHR_buffer_device_address |= support_KHR_buffer_device_address
         ;
   info->have_EXT_descriptor_buffer |= support_EXT_descriptor_buffer
         ;
   info->have_EXT_conditional_rendering |= support_EXT_conditional_rendering
         && (info->cond_render_feats.conditionalRendering);
   info->have_EXT_transform_feedback |= support_EXT_transform_feedback
         && (info->tf_feats.transformFeedback);
   info->have_EXT_index_type_uint8 |= support_EXT_index_type_uint8
         && (info->index_uint8_feats.indexTypeUint8);
   info->have_KHR_image_format_list |= support_KHR_image_format_list
         ;
   info->have_KHR_sampler_ycbcr_conversion |= support_KHR_sampler_ycbcr_conversion
         ;
   info->have_KHR_imageless_framebuffer |= support_KHR_imageless_framebuffer
         ;
   info->have_EXT_robustness2 |= support_EXT_robustness2
         && (info->rb2_feats.nullDescriptor);
   info->have_EXT_image_drm_format_modifier |= support_EXT_image_drm_format_modifier
         ;
   info->have_EXT_vertex_attribute_divisor |= support_EXT_vertex_attribute_divisor
         && (info->vdiv_feats.vertexAttributeInstanceRateDivisor);
   info->have_EXT_calibrated_timestamps |= support_EXT_calibrated_timestamps
         ;
   info->have_NV_linear_color_attachment |= support_NV_linear_color_attachment
         ;
   info->have_KHR_dynamic_rendering |= support_KHR_dynamic_rendering
         ;
   info->have_KHR_shader_clock |= support_KHR_shader_clock
         && (info->shader_clock_feats.shaderSubgroupClock);
   info->have_EXT_sampler_filter_minmax |= support_EXT_sampler_filter_minmax
         ;
   info->have_EXT_custom_border_color |= support_EXT_custom_border_color
         && (info->border_color_feats.customBorderColors);
   info->have_EXT_non_seamless_cube_map |= support_EXT_non_seamless_cube_map
         ;
   info->have_EXT_border_color_swizzle |= support_EXT_border_color_swizzle
         ;
   info->have_EXT_blend_operation_advanced |= support_EXT_blend_operation_advanced
         && (info->blend_props.advancedBlendNonPremultipliedSrcColor)
&& (info->blend_props.advancedBlendNonPremultipliedDstColor);
   info->have_EXT_extended_dynamic_state |= support_EXT_extended_dynamic_state
         && (info->dynamic_state_feats.extendedDynamicState);
   info->have_EXT_extended_dynamic_state2 |= support_EXT_extended_dynamic_state2
         && (info->dynamic_state2_feats.extendedDynamicState2);
   info->have_EXT_pipeline_creation_cache_control |= support_EXT_pipeline_creation_cache_control
         && (info->pipeline_cache_control_feats.pipelineCreationCacheControl);
   info->have_EXT_shader_stencil_export |= support_EXT_shader_stencil_export
         ;
#ifdef VK_KHR_PORTABILITY_SUBSET_EXTENSION_NAME
   info->have_KHR_portability_subset |= support_KHR_portability_subset
         ;
#endif
   info->have_NV_compute_shader_derivatives |= support_NV_compute_shader_derivatives
         && (info->shader_derivs_feats.computeDerivativeGroupQuads)
&& (info->shader_derivs_feats.computeDerivativeGroupLinear);
   info->have_KHR_timeline_semaphore |= support_KHR_timeline_semaphore
         ;
   info->have_EXT_color_write_enable |= support_EXT_color_write_enable
         ;
   info->have_EXT_4444_formats |= support_EXT_4444_formats
         ;
   info->have_EXT_scalar_block_layout |= support_EXT_scalar_block_layout
         && (info->scalar_block_layout_feats.scalarBlockLayout);
   info->have_KHR_swapchain |= support_KHR_swapchain
         ;
   info->have_EXT_rasterization_order_attachment_access |= support_EXT_rasterization_order_attachment_access
         && (info->rast_order_access_feats.rasterizationOrderColorAttachmentAccess);
   info->have_KHR_shader_float16_int8 |= support_KHR_shader_float16_int8
         ;
   info->have_EXT_multi_draw |= support_EXT_multi_draw
         && (info->multidraw_feats.multiDraw);
   info->have_EXT_primitives_generated_query |= support_EXT_primitives_generated_query
         ;
   info->have_KHR_pipeline_library |= support_KHR_pipeline_library
         ;
   info->have_EXT_graphics_pipeline_library |= support_EXT_graphics_pipeline_library
         && (info->gpl_props.graphicsPipelineLibraryFastLinking);
   info->have_KHR_push_descriptor |= support_KHR_push_descriptor
         ;
   info->have_KHR_descriptor_update_template |= support_KHR_descriptor_update_template
         ;
   info->have_EXT_line_rasterization |= support_EXT_line_rasterization
         ;
   info->have_EXT_mutable_descriptor_type |= support_EXT_mutable_descriptor_type
         ;
   info->have_EXT_vertex_input_dynamic_state |= support_EXT_vertex_input_dynamic_state
         && (info->vertex_input_feats.vertexInputDynamicState);
   info->have_EXT_primitive_topology_list_restart |= support_EXT_primitive_topology_list_restart
         && (info->list_restart_feats.primitiveTopologyListRestart);
   info->have_KHR_dedicated_allocation |= support_KHR_dedicated_allocation
         ;
   info->have_EXT_descriptor_indexing |= support_EXT_descriptor_indexing
         && (info->desc_indexing_feats.descriptorBindingPartiallyBound);
   info->have_EXT_depth_clip_enable |= support_EXT_depth_clip_enable
         ;
   info->have_EXT_shader_demote_to_helper_invocation |= support_EXT_shader_demote_to_helper_invocation
         ;
   info->have_KHR_shader_terminate_invocation |= support_KHR_shader_terminate_invocation
         ;
   info->have_EXT_host_image_copy |= support_EXT_host_image_copy
         ;
   }

   // generate extension list
   num_extensions = 0;

   if (info->have_KHR_maintenance1) {
       info->extensions[num_extensions++] = "VK_KHR_maintenance1";
   } else {
       fprintf(stderr, "VK: VK_KHR_maintenance1 required!\n");
       goto fail;
   }
   if (info->have_KHR_maintenance2) {
       info->extensions[num_extensions++] = "VK_KHR_maintenance2";
   }
   if (info->have_KHR_maintenance3) {
       info->extensions[num_extensions++] = "VK_KHR_maintenance3";
   }
   if (info->have_KHR_maintenance4) {
       info->extensions[num_extensions++] = "VK_KHR_maintenance4";
   }
   if (info->have_KHR_external_memory) {
       info->extensions[num_extensions++] = "VK_KHR_external_memory";
   }
   if (info->have_KHR_external_memory_fd) {
       info->extensions[num_extensions++] = "VK_KHR_external_memory_fd";
   }
   if (info->have_KHR_vulkan_memory_model) {
       info->extensions[num_extensions++] = "VK_KHR_vulkan_memory_model";
   }
   if (info->have_KHR_external_semaphore_fd) {
       info->extensions[num_extensions++] = "VK_KHR_external_semaphore_fd";
   }
   if (info->have_KHR_create_renderpass2) {
       info->extensions[num_extensions++] = "VK_KHR_create_renderpass2";
   } else {
       fprintf(stderr, "VK: VK_KHR_create_renderpass2 required!\n");
       goto fail;
   }
   if (info->have_KHR_synchronization2) {
       info->extensions[num_extensions++] = "VK_KHR_synchronization2";
   }
   if (info->have_KHR_swapchain_mutable_format) {
       info->extensions[num_extensions++] = "VK_KHR_swapchain_mutable_format";
   }
   if (info->have_EXT_provoking_vertex) {
       info->extensions[num_extensions++] = "VK_EXT_provoking_vertex";
   }
   if (info->have_EXT_shader_viewport_index_layer) {
       info->extensions[num_extensions++] = "VK_EXT_shader_viewport_index_layer";
   }
   if (info->have_KHR_get_memory_requirements2) {
       info->extensions[num_extensions++] = "VK_KHR_get_memory_requirements2";
   }
   if (info->have_EXT_post_depth_coverage) {
       info->extensions[num_extensions++] = "VK_EXT_post_depth_coverage";
   }
   if (info->have_EXT_depth_clip_control) {
       info->extensions[num_extensions++] = "VK_EXT_depth_clip_control";
   }
   if (info->have_EXT_shader_subgroup_ballot) {
       info->extensions[num_extensions++] = "VK_EXT_shader_subgroup_ballot";
   }
   if (info->have_EXT_shader_subgroup_vote) {
       info->extensions[num_extensions++] = "VK_EXT_shader_subgroup_vote";
   }
   if (info->have_EXT_shader_atomic_float) {
       info->extensions[num_extensions++] = "VK_EXT_shader_atomic_float";
   }
   if (info->have_KHR_shader_atomic_int64) {
       info->extensions[num_extensions++] = "VK_KHR_shader_atomic_int64";
   }
   if (info->have_KHR_8bit_storage) {
       info->extensions[num_extensions++] = "VK_KHR_8bit_storage";
   }
   if (info->have_KHR_16bit_storage) {
       info->extensions[num_extensions++] = "VK_KHR_16bit_storage";
   }
   if (info->have_EXT_image_2d_view_of_3d) {
       info->extensions[num_extensions++] = "VK_EXT_image_2d_view_of_3d";
   }
   if (info->have_KHR_driver_properties) {
       info->extensions[num_extensions++] = "VK_KHR_driver_properties";
   }
   if (info->have_EXT_memory_budget) {
       info->extensions[num_extensions++] = "VK_EXT_memory_budget";
   }
   if (info->have_KHR_draw_indirect_count) {
       info->extensions[num_extensions++] = "VK_KHR_draw_indirect_count";
   }
   if (info->have_EXT_fragment_shader_interlock) {
       info->extensions[num_extensions++] = "VK_EXT_fragment_shader_interlock";
   }
   if (info->have_EXT_sample_locations) {
       info->extensions[num_extensions++] = "VK_EXT_sample_locations";
   }
   if (info->have_EXT_conservative_rasterization) {
       info->extensions[num_extensions++] = "VK_EXT_conservative_rasterization";
   }
   if (info->have_KHR_shader_draw_parameters) {
       info->extensions[num_extensions++] = "VK_KHR_shader_draw_parameters";
   }
   if (info->have_KHR_sampler_mirror_clamp_to_edge) {
       info->extensions[num_extensions++] = "VK_KHR_sampler_mirror_clamp_to_edge";
   }
   if (info->have_KHR_buffer_device_address) {
       info->extensions[num_extensions++] = "VK_KHR_buffer_device_address";
   }
   if (info->have_EXT_descriptor_buffer) {
       info->extensions[num_extensions++] = "VK_EXT_descriptor_buffer";
   }
   if (info->have_EXT_conditional_rendering) {
       info->extensions[num_extensions++] = "VK_EXT_conditional_rendering";
   }
   if (info->have_EXT_transform_feedback) {
       info->extensions[num_extensions++] = "VK_EXT_transform_feedback";
   }
   if (info->have_EXT_index_type_uint8) {
       info->extensions[num_extensions++] = "VK_EXT_index_type_uint8";
   }
   if (info->have_KHR_image_format_list) {
       info->extensions[num_extensions++] = "VK_KHR_image_format_list";
   }
   if (info->have_KHR_sampler_ycbcr_conversion) {
       info->extensions[num_extensions++] = "VK_KHR_sampler_ycbcr_conversion";
   }
   if (info->have_KHR_imageless_framebuffer) {
       info->extensions[num_extensions++] = "VK_KHR_imageless_framebuffer";
   }
   if (info->have_EXT_robustness2) {
       info->extensions[num_extensions++] = "VK_EXT_robustness2";
   }
   if (info->have_EXT_image_drm_format_modifier) {
       info->extensions[num_extensions++] = "VK_EXT_image_drm_format_modifier";
   }
   if (info->have_EXT_vertex_attribute_divisor) {
       info->extensions[num_extensions++] = "VK_EXT_vertex_attribute_divisor";
   }
   if (info->have_EXT_calibrated_timestamps) {
       info->extensions[num_extensions++] = "VK_EXT_calibrated_timestamps";
   }
   if (info->have_NV_linear_color_attachment) {
       info->extensions[num_extensions++] = "VK_NV_linear_color_attachment";
   }
   if (info->have_KHR_dynamic_rendering) {
       info->extensions[num_extensions++] = "VK_KHR_dynamic_rendering";
   }
   if (info->have_KHR_shader_clock) {
       info->extensions[num_extensions++] = "VK_KHR_shader_clock";
   }
   if (info->have_EXT_sampler_filter_minmax) {
       info->extensions[num_extensions++] = "VK_EXT_sampler_filter_minmax";
   }
   if (info->have_EXT_custom_border_color) {
       info->extensions[num_extensions++] = "VK_EXT_custom_border_color";
   }
   if (info->have_EXT_non_seamless_cube_map) {
       info->extensions[num_extensions++] = "VK_EXT_non_seamless_cube_map";
   }
   if (info->have_EXT_border_color_swizzle) {
       info->extensions[num_extensions++] = "VK_EXT_border_color_swizzle";
   }
   if (info->have_EXT_blend_operation_advanced) {
       info->extensions[num_extensions++] = "VK_EXT_blend_operation_advanced";
   }
   if (info->have_EXT_extended_dynamic_state) {
       info->extensions[num_extensions++] = "VK_EXT_extended_dynamic_state";
   }
   if (info->have_EXT_extended_dynamic_state2) {
       info->extensions[num_extensions++] = "VK_EXT_extended_dynamic_state2";
   }
   if (info->have_EXT_pipeline_creation_cache_control) {
       info->extensions[num_extensions++] = "VK_EXT_pipeline_creation_cache_control";
   }
   if (info->have_EXT_shader_stencil_export) {
       info->extensions[num_extensions++] = "VK_EXT_shader_stencil_export";
   }
#ifdef VK_KHR_PORTABILITY_SUBSET_EXTENSION_NAME
   if (info->have_KHR_portability_subset) {
       info->extensions[num_extensions++] = "VK_KHR_portability_subset";
   }
#endif
   if (info->have_NV_compute_shader_derivatives) {
       info->extensions[num_extensions++] = "VK_NV_compute_shader_derivatives";
   }
   if (info->have_KHR_timeline_semaphore) {
       info->extensions[num_extensions++] = "VK_KHR_timeline_semaphore";
   }
   if (info->have_EXT_color_write_enable) {
       info->extensions[num_extensions++] = "VK_EXT_color_write_enable";
   }
   if (info->have_EXT_4444_formats) {
       info->extensions[num_extensions++] = "VK_EXT_4444_formats";
   }
   if (info->have_EXT_scalar_block_layout) {
       info->extensions[num_extensions++] = "VK_EXT_scalar_block_layout";
   }
   if (info->have_KHR_swapchain) {
       info->extensions[num_extensions++] = "VK_KHR_swapchain";
   }
   if (info->have_EXT_rasterization_order_attachment_access) {
       info->extensions[num_extensions++] = "VK_EXT_rasterization_order_attachment_access";
   }
   if (info->have_KHR_shader_float16_int8) {
       info->extensions[num_extensions++] = "VK_KHR_shader_float16_int8";
   }
   if (info->have_EXT_multi_draw) {
       info->extensions[num_extensions++] = "VK_EXT_multi_draw";
   }
   if (info->have_EXT_primitives_generated_query) {
       info->extensions[num_extensions++] = "VK_EXT_primitives_generated_query";
   }
   if (info->have_KHR_pipeline_library) {
       info->extensions[num_extensions++] = "VK_KHR_pipeline_library";
   }
   if (info->have_EXT_graphics_pipeline_library) {
       info->extensions[num_extensions++] = "VK_EXT_graphics_pipeline_library";
   }
   if (info->have_KHR_push_descriptor) {
       info->extensions[num_extensions++] = "VK_KHR_push_descriptor";
   }
   if (info->have_KHR_descriptor_update_template) {
       info->extensions[num_extensions++] = "VK_KHR_descriptor_update_template";
   }
   if (info->have_EXT_line_rasterization) {
       info->extensions[num_extensions++] = "VK_EXT_line_rasterization";
   }
   if (info->have_EXT_mutable_descriptor_type) {
       info->extensions[num_extensions++] = "VK_EXT_mutable_descriptor_type";
   }
   if (info->have_EXT_vertex_input_dynamic_state) {
       info->extensions[num_extensions++] = "VK_EXT_vertex_input_dynamic_state";
   }
   if (info->have_EXT_primitive_topology_list_restart) {
       info->extensions[num_extensions++] = "VK_EXT_primitive_topology_list_restart";
   }
   if (info->have_KHR_dedicated_allocation) {
       info->extensions[num_extensions++] = "VK_KHR_dedicated_allocation";
   }
   if (info->have_EXT_descriptor_indexing) {
       info->extensions[num_extensions++] = "VK_EXT_descriptor_indexing";
   }
   if (info->have_EXT_depth_clip_enable) {
       info->extensions[num_extensions++] = "VK_EXT_depth_clip_enable";
   }
   if (info->have_EXT_shader_demote_to_helper_invocation) {
       info->extensions[num_extensions++] = "VK_EXT_shader_demote_to_helper_invocation";
   }
   if (info->have_KHR_shader_terminate_invocation) {
       info->extensions[num_extensions++] = "VK_KHR_shader_terminate_invocation";
   }
   if (info->have_EXT_host_image_copy) {
       info->extensions[num_extensions++] = "VK_EXT_host_image_copy";
   }

   info->num_extensions = num_extensions;

   return true;

fail:
   return false;
}

void
vk_verify_device_extensions(struct vk_device *dev)
{
   if (dev->info.have_KHR_maintenance1) {
#ifdef _WIN32
      if (!dev->vk.TrimCommandPoolKHR) {
#ifndef NDEBUG
         dev->vk.TrimCommandPoolKHR = (PFN_vkTrimCommandPoolKHR)vk_stub_TrimCommandPoolKHR;
#else
         dev->vk.TrimCommandPoolKHR = (PFN_vkTrimCommandPoolKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_KHR_maintenance2) {
   }
   if (dev->info.have_KHR_maintenance3) {
#ifdef _WIN32
      if (!dev->vk.GetDescriptorSetLayoutSupportKHR) {
#ifndef NDEBUG
         dev->vk.GetDescriptorSetLayoutSupportKHR = (PFN_vkGetDescriptorSetLayoutSupportKHR)vk_stub_GetDescriptorSetLayoutSupportKHR;
#else
         dev->vk.GetDescriptorSetLayoutSupportKHR = (PFN_vkGetDescriptorSetLayoutSupportKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_KHR_maintenance4) {
#ifdef _WIN32
      if (!dev->vk.GetDeviceBufferMemoryRequirementsKHR) {
#ifndef NDEBUG
         dev->vk.GetDeviceBufferMemoryRequirementsKHR = (PFN_vkGetDeviceBufferMemoryRequirementsKHR)vk_stub_GetDeviceBufferMemoryRequirementsKHR;
#else
         dev->vk.GetDeviceBufferMemoryRequirementsKHR = (PFN_vkGetDeviceBufferMemoryRequirementsKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetDeviceImageMemoryRequirementsKHR) {
#ifndef NDEBUG
         dev->vk.GetDeviceImageMemoryRequirementsKHR = (PFN_vkGetDeviceImageMemoryRequirementsKHR)vk_stub_GetDeviceImageMemoryRequirementsKHR;
#else
         dev->vk.GetDeviceImageMemoryRequirementsKHR = (PFN_vkGetDeviceImageMemoryRequirementsKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetDeviceImageSparseMemoryRequirementsKHR) {
#ifndef NDEBUG
         dev->vk.GetDeviceImageSparseMemoryRequirementsKHR = (PFN_vkGetDeviceImageSparseMemoryRequirementsKHR)vk_stub_GetDeviceImageSparseMemoryRequirementsKHR;
#else
         dev->vk.GetDeviceImageSparseMemoryRequirementsKHR = (PFN_vkGetDeviceImageSparseMemoryRequirementsKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_KHR_external_memory) {
   }
   if (dev->info.have_KHR_external_memory_fd) {
#ifdef _WIN32
      if (!dev->vk.GetMemoryFdKHR) {
#ifndef NDEBUG
         dev->vk.GetMemoryFdKHR = (PFN_vkGetMemoryFdKHR)vk_stub_GetMemoryFdKHR;
#else
         dev->vk.GetMemoryFdKHR = (PFN_vkGetMemoryFdKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetMemoryFdPropertiesKHR) {
#ifndef NDEBUG
         dev->vk.GetMemoryFdPropertiesKHR = (PFN_vkGetMemoryFdPropertiesKHR)vk_stub_GetMemoryFdPropertiesKHR;
#else
         dev->vk.GetMemoryFdPropertiesKHR = (PFN_vkGetMemoryFdPropertiesKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_KHR_vulkan_memory_model) {
   }
   if (dev->info.have_KHR_external_semaphore_fd) {
#ifdef _WIN32
      if (!dev->vk.ImportSemaphoreFdKHR) {
#ifndef NDEBUG
         dev->vk.ImportSemaphoreFdKHR = (PFN_vkImportSemaphoreFdKHR)vk_stub_ImportSemaphoreFdKHR;
#else
         dev->vk.ImportSemaphoreFdKHR = (PFN_vkImportSemaphoreFdKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetSemaphoreFdKHR) {
#ifndef NDEBUG
         dev->vk.GetSemaphoreFdKHR = (PFN_vkGetSemaphoreFdKHR)vk_stub_GetSemaphoreFdKHR;
#else
         dev->vk.GetSemaphoreFdKHR = (PFN_vkGetSemaphoreFdKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_KHR_create_renderpass2) {
#ifdef _WIN32
      if (!dev->vk.CreateRenderPass2KHR) {
#ifndef NDEBUG
         dev->vk.CreateRenderPass2KHR = (PFN_vkCreateRenderPass2KHR)vk_stub_CreateRenderPass2KHR;
#else
         dev->vk.CreateRenderPass2KHR = (PFN_vkCreateRenderPass2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdBeginRenderPass2KHR) {
#ifndef NDEBUG
         dev->vk.CmdBeginRenderPass2KHR = (PFN_vkCmdBeginRenderPass2KHR)vk_stub_CmdBeginRenderPass2KHR;
#else
         dev->vk.CmdBeginRenderPass2KHR = (PFN_vkCmdBeginRenderPass2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdNextSubpass2KHR) {
#ifndef NDEBUG
         dev->vk.CmdNextSubpass2KHR = (PFN_vkCmdNextSubpass2KHR)vk_stub_CmdNextSubpass2KHR;
#else
         dev->vk.CmdNextSubpass2KHR = (PFN_vkCmdNextSubpass2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdEndRenderPass2KHR) {
#ifndef NDEBUG
         dev->vk.CmdEndRenderPass2KHR = (PFN_vkCmdEndRenderPass2KHR)vk_stub_CmdEndRenderPass2KHR;
#else
         dev->vk.CmdEndRenderPass2KHR = (PFN_vkCmdEndRenderPass2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_KHR_synchronization2) {
#ifdef _WIN32
      if (!dev->vk.CmdSetEvent2KHR) {
#ifndef NDEBUG
         dev->vk.CmdSetEvent2KHR = (PFN_vkCmdSetEvent2KHR)vk_stub_CmdSetEvent2KHR;
#else
         dev->vk.CmdSetEvent2KHR = (PFN_vkCmdSetEvent2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdResetEvent2KHR) {
#ifndef NDEBUG
         dev->vk.CmdResetEvent2KHR = (PFN_vkCmdResetEvent2KHR)vk_stub_CmdResetEvent2KHR;
#else
         dev->vk.CmdResetEvent2KHR = (PFN_vkCmdResetEvent2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdWaitEvents2KHR) {
#ifndef NDEBUG
         dev->vk.CmdWaitEvents2KHR = (PFN_vkCmdWaitEvents2KHR)vk_stub_CmdWaitEvents2KHR;
#else
         dev->vk.CmdWaitEvents2KHR = (PFN_vkCmdWaitEvents2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdPipelineBarrier2KHR) {
#ifndef NDEBUG
         dev->vk.CmdPipelineBarrier2KHR = (PFN_vkCmdPipelineBarrier2KHR)vk_stub_CmdPipelineBarrier2KHR;
#else
         dev->vk.CmdPipelineBarrier2KHR = (PFN_vkCmdPipelineBarrier2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdWriteTimestamp2KHR) {
#ifndef NDEBUG
         dev->vk.CmdWriteTimestamp2KHR = (PFN_vkCmdWriteTimestamp2KHR)vk_stub_CmdWriteTimestamp2KHR;
#else
         dev->vk.CmdWriteTimestamp2KHR = (PFN_vkCmdWriteTimestamp2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.QueueSubmit2KHR) {
#ifndef NDEBUG
         dev->vk.QueueSubmit2KHR = (PFN_vkQueueSubmit2KHR)vk_stub_QueueSubmit2KHR;
#else
         dev->vk.QueueSubmit2KHR = (PFN_vkQueueSubmit2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdWriteBufferMarker2AMD) {
#ifndef NDEBUG
         dev->vk.CmdWriteBufferMarker2AMD = (PFN_vkCmdWriteBufferMarker2AMD)vk_stub_CmdWriteBufferMarker2AMD;
#else
         dev->vk.CmdWriteBufferMarker2AMD = (PFN_vkCmdWriteBufferMarker2AMD)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetQueueCheckpointData2NV) {
#ifndef NDEBUG
         dev->vk.GetQueueCheckpointData2NV = (PFN_vkGetQueueCheckpointData2NV)vk_stub_GetQueueCheckpointData2NV;
#else
         dev->vk.GetQueueCheckpointData2NV = (PFN_vkGetQueueCheckpointData2NV)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_KHR_swapchain_mutable_format) {
   }
   if (dev->info.have_EXT_provoking_vertex) {
   }
   if (dev->info.have_EXT_shader_viewport_index_layer) {
   }
   if (dev->info.have_KHR_get_memory_requirements2) {
#ifdef _WIN32
      if (!dev->vk.GetImageMemoryRequirements2KHR) {
#ifndef NDEBUG
         dev->vk.GetImageMemoryRequirements2KHR = (PFN_vkGetImageMemoryRequirements2KHR)vk_stub_GetImageMemoryRequirements2KHR;
#else
         dev->vk.GetImageMemoryRequirements2KHR = (PFN_vkGetImageMemoryRequirements2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetBufferMemoryRequirements2KHR) {
#ifndef NDEBUG
         dev->vk.GetBufferMemoryRequirements2KHR = (PFN_vkGetBufferMemoryRequirements2KHR)vk_stub_GetBufferMemoryRequirements2KHR;
#else
         dev->vk.GetBufferMemoryRequirements2KHR = (PFN_vkGetBufferMemoryRequirements2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetImageSparseMemoryRequirements2KHR) {
#ifndef NDEBUG
         dev->vk.GetImageSparseMemoryRequirements2KHR = (PFN_vkGetImageSparseMemoryRequirements2KHR)vk_stub_GetImageSparseMemoryRequirements2KHR;
#else
         dev->vk.GetImageSparseMemoryRequirements2KHR = (PFN_vkGetImageSparseMemoryRequirements2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_post_depth_coverage) {
   }
   if (dev->info.have_EXT_depth_clip_control) {
   }
   if (dev->info.have_EXT_shader_subgroup_ballot) {
   }
   if (dev->info.have_EXT_shader_subgroup_vote) {
   }
   if (dev->info.have_EXT_shader_atomic_float) {
   }
   if (dev->info.have_KHR_shader_atomic_int64) {
   }
   if (dev->info.have_KHR_8bit_storage) {
   }
   if (dev->info.have_KHR_16bit_storage) {
   }
   if (dev->info.have_EXT_image_2d_view_of_3d) {
   }
   if (dev->info.have_KHR_driver_properties) {
   }
   if (dev->info.have_EXT_memory_budget) {
   }
   if (dev->info.have_KHR_draw_indirect_count) {
#ifdef _WIN32
      if (!dev->vk.CmdDrawIndirectCountKHR) {
#ifndef NDEBUG
         dev->vk.CmdDrawIndirectCountKHR = (PFN_vkCmdDrawIndirectCountKHR)vk_stub_CmdDrawIndirectCountKHR;
#else
         dev->vk.CmdDrawIndirectCountKHR = (PFN_vkCmdDrawIndirectCountKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdDrawIndexedIndirectCountKHR) {
#ifndef NDEBUG
         dev->vk.CmdDrawIndexedIndirectCountKHR = (PFN_vkCmdDrawIndexedIndirectCountKHR)vk_stub_CmdDrawIndexedIndirectCountKHR;
#else
         dev->vk.CmdDrawIndexedIndirectCountKHR = (PFN_vkCmdDrawIndexedIndirectCountKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_fragment_shader_interlock) {
   }
   if (dev->info.have_EXT_sample_locations) {
#ifdef _WIN32
      if (!dev->vk.CmdSetSampleLocationsEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetSampleLocationsEXT = (PFN_vkCmdSetSampleLocationsEXT)vk_stub_CmdSetSampleLocationsEXT;
#else
         dev->vk.CmdSetSampleLocationsEXT = (PFN_vkCmdSetSampleLocationsEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_conservative_rasterization) {
   }
   if (dev->info.have_KHR_shader_draw_parameters) {
   }
   if (dev->info.have_KHR_sampler_mirror_clamp_to_edge) {
   }
   if (dev->info.have_KHR_buffer_device_address) {
#ifdef _WIN32
      if (!dev->vk.GetBufferDeviceAddressKHR) {
#ifndef NDEBUG
         dev->vk.GetBufferDeviceAddressKHR = (PFN_vkGetBufferDeviceAddressKHR)vk_stub_GetBufferDeviceAddressKHR;
#else
         dev->vk.GetBufferDeviceAddressKHR = (PFN_vkGetBufferDeviceAddressKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetBufferOpaqueCaptureAddressKHR) {
#ifndef NDEBUG
         dev->vk.GetBufferOpaqueCaptureAddressKHR = (PFN_vkGetBufferOpaqueCaptureAddressKHR)vk_stub_GetBufferOpaqueCaptureAddressKHR;
#else
         dev->vk.GetBufferOpaqueCaptureAddressKHR = (PFN_vkGetBufferOpaqueCaptureAddressKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetDeviceMemoryOpaqueCaptureAddressKHR) {
#ifndef NDEBUG
         dev->vk.GetDeviceMemoryOpaqueCaptureAddressKHR = (PFN_vkGetDeviceMemoryOpaqueCaptureAddressKHR)vk_stub_GetDeviceMemoryOpaqueCaptureAddressKHR;
#else
         dev->vk.GetDeviceMemoryOpaqueCaptureAddressKHR = (PFN_vkGetDeviceMemoryOpaqueCaptureAddressKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_descriptor_buffer) {
#ifdef _WIN32
      if (!dev->vk.GetDescriptorSetLayoutSizeEXT) {
#ifndef NDEBUG
         dev->vk.GetDescriptorSetLayoutSizeEXT = (PFN_vkGetDescriptorSetLayoutSizeEXT)vk_stub_GetDescriptorSetLayoutSizeEXT;
#else
         dev->vk.GetDescriptorSetLayoutSizeEXT = (PFN_vkGetDescriptorSetLayoutSizeEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetDescriptorSetLayoutBindingOffsetEXT) {
#ifndef NDEBUG
         dev->vk.GetDescriptorSetLayoutBindingOffsetEXT = (PFN_vkGetDescriptorSetLayoutBindingOffsetEXT)vk_stub_GetDescriptorSetLayoutBindingOffsetEXT;
#else
         dev->vk.GetDescriptorSetLayoutBindingOffsetEXT = (PFN_vkGetDescriptorSetLayoutBindingOffsetEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetDescriptorEXT) {
#ifndef NDEBUG
         dev->vk.GetDescriptorEXT = (PFN_vkGetDescriptorEXT)vk_stub_GetDescriptorEXT;
#else
         dev->vk.GetDescriptorEXT = (PFN_vkGetDescriptorEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdBindDescriptorBuffersEXT) {
#ifndef NDEBUG
         dev->vk.CmdBindDescriptorBuffersEXT = (PFN_vkCmdBindDescriptorBuffersEXT)vk_stub_CmdBindDescriptorBuffersEXT;
#else
         dev->vk.CmdBindDescriptorBuffersEXT = (PFN_vkCmdBindDescriptorBuffersEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetDescriptorBufferOffsetsEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetDescriptorBufferOffsetsEXT = (PFN_vkCmdSetDescriptorBufferOffsetsEXT)vk_stub_CmdSetDescriptorBufferOffsetsEXT;
#else
         dev->vk.CmdSetDescriptorBufferOffsetsEXT = (PFN_vkCmdSetDescriptorBufferOffsetsEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdBindDescriptorBufferEmbeddedSamplersEXT) {
#ifndef NDEBUG
         dev->vk.CmdBindDescriptorBufferEmbeddedSamplersEXT = (PFN_vkCmdBindDescriptorBufferEmbeddedSamplersEXT)vk_stub_CmdBindDescriptorBufferEmbeddedSamplersEXT;
#else
         dev->vk.CmdBindDescriptorBufferEmbeddedSamplersEXT = (PFN_vkCmdBindDescriptorBufferEmbeddedSamplersEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetBufferOpaqueCaptureDescriptorDataEXT) {
#ifndef NDEBUG
         dev->vk.GetBufferOpaqueCaptureDescriptorDataEXT = (PFN_vkGetBufferOpaqueCaptureDescriptorDataEXT)vk_stub_GetBufferOpaqueCaptureDescriptorDataEXT;
#else
         dev->vk.GetBufferOpaqueCaptureDescriptorDataEXT = (PFN_vkGetBufferOpaqueCaptureDescriptorDataEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetImageOpaqueCaptureDescriptorDataEXT) {
#ifndef NDEBUG
         dev->vk.GetImageOpaqueCaptureDescriptorDataEXT = (PFN_vkGetImageOpaqueCaptureDescriptorDataEXT)vk_stub_GetImageOpaqueCaptureDescriptorDataEXT;
#else
         dev->vk.GetImageOpaqueCaptureDescriptorDataEXT = (PFN_vkGetImageOpaqueCaptureDescriptorDataEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetImageViewOpaqueCaptureDescriptorDataEXT) {
#ifndef NDEBUG
         dev->vk.GetImageViewOpaqueCaptureDescriptorDataEXT = (PFN_vkGetImageViewOpaqueCaptureDescriptorDataEXT)vk_stub_GetImageViewOpaqueCaptureDescriptorDataEXT;
#else
         dev->vk.GetImageViewOpaqueCaptureDescriptorDataEXT = (PFN_vkGetImageViewOpaqueCaptureDescriptorDataEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetSamplerOpaqueCaptureDescriptorDataEXT) {
#ifndef NDEBUG
         dev->vk.GetSamplerOpaqueCaptureDescriptorDataEXT = (PFN_vkGetSamplerOpaqueCaptureDescriptorDataEXT)vk_stub_GetSamplerOpaqueCaptureDescriptorDataEXT;
#else
         dev->vk.GetSamplerOpaqueCaptureDescriptorDataEXT = (PFN_vkGetSamplerOpaqueCaptureDescriptorDataEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetAccelerationStructureOpaqueCaptureDescriptorDataEXT) {
#ifndef NDEBUG
         dev->vk.GetAccelerationStructureOpaqueCaptureDescriptorDataEXT = (PFN_vkGetAccelerationStructureOpaqueCaptureDescriptorDataEXT)vk_stub_GetAccelerationStructureOpaqueCaptureDescriptorDataEXT;
#else
         dev->vk.GetAccelerationStructureOpaqueCaptureDescriptorDataEXT = (PFN_vkGetAccelerationStructureOpaqueCaptureDescriptorDataEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_conditional_rendering) {
#ifdef _WIN32
      if (!dev->vk.CmdBeginConditionalRenderingEXT) {
#ifndef NDEBUG
         dev->vk.CmdBeginConditionalRenderingEXT = (PFN_vkCmdBeginConditionalRenderingEXT)vk_stub_CmdBeginConditionalRenderingEXT;
#else
         dev->vk.CmdBeginConditionalRenderingEXT = (PFN_vkCmdBeginConditionalRenderingEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdEndConditionalRenderingEXT) {
#ifndef NDEBUG
         dev->vk.CmdEndConditionalRenderingEXT = (PFN_vkCmdEndConditionalRenderingEXT)vk_stub_CmdEndConditionalRenderingEXT;
#else
         dev->vk.CmdEndConditionalRenderingEXT = (PFN_vkCmdEndConditionalRenderingEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_transform_feedback) {
#ifdef _WIN32
      if (!dev->vk.CmdBindTransformFeedbackBuffersEXT) {
#ifndef NDEBUG
         dev->vk.CmdBindTransformFeedbackBuffersEXT = (PFN_vkCmdBindTransformFeedbackBuffersEXT)vk_stub_CmdBindTransformFeedbackBuffersEXT;
#else
         dev->vk.CmdBindTransformFeedbackBuffersEXT = (PFN_vkCmdBindTransformFeedbackBuffersEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdBeginTransformFeedbackEXT) {
#ifndef NDEBUG
         dev->vk.CmdBeginTransformFeedbackEXT = (PFN_vkCmdBeginTransformFeedbackEXT)vk_stub_CmdBeginTransformFeedbackEXT;
#else
         dev->vk.CmdBeginTransformFeedbackEXT = (PFN_vkCmdBeginTransformFeedbackEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdEndTransformFeedbackEXT) {
#ifndef NDEBUG
         dev->vk.CmdEndTransformFeedbackEXT = (PFN_vkCmdEndTransformFeedbackEXT)vk_stub_CmdEndTransformFeedbackEXT;
#else
         dev->vk.CmdEndTransformFeedbackEXT = (PFN_vkCmdEndTransformFeedbackEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdBeginQueryIndexedEXT) {
#ifndef NDEBUG
         dev->vk.CmdBeginQueryIndexedEXT = (PFN_vkCmdBeginQueryIndexedEXT)vk_stub_CmdBeginQueryIndexedEXT;
#else
         dev->vk.CmdBeginQueryIndexedEXT = (PFN_vkCmdBeginQueryIndexedEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdEndQueryIndexedEXT) {
#ifndef NDEBUG
         dev->vk.CmdEndQueryIndexedEXT = (PFN_vkCmdEndQueryIndexedEXT)vk_stub_CmdEndQueryIndexedEXT;
#else
         dev->vk.CmdEndQueryIndexedEXT = (PFN_vkCmdEndQueryIndexedEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdDrawIndirectByteCountEXT) {
#ifndef NDEBUG
         dev->vk.CmdDrawIndirectByteCountEXT = (PFN_vkCmdDrawIndirectByteCountEXT)vk_stub_CmdDrawIndirectByteCountEXT;
#else
         dev->vk.CmdDrawIndirectByteCountEXT = (PFN_vkCmdDrawIndirectByteCountEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_index_type_uint8) {
   }
   if (dev->info.have_KHR_image_format_list) {
   }
   if (dev->info.have_KHR_sampler_ycbcr_conversion) {
#ifdef _WIN32
      if (!dev->vk.CreateSamplerYcbcrConversionKHR) {
#ifndef NDEBUG
         dev->vk.CreateSamplerYcbcrConversionKHR = (PFN_vkCreateSamplerYcbcrConversionKHR)vk_stub_CreateSamplerYcbcrConversionKHR;
#else
         dev->vk.CreateSamplerYcbcrConversionKHR = (PFN_vkCreateSamplerYcbcrConversionKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.DestroySamplerYcbcrConversionKHR) {
#ifndef NDEBUG
         dev->vk.DestroySamplerYcbcrConversionKHR = (PFN_vkDestroySamplerYcbcrConversionKHR)vk_stub_DestroySamplerYcbcrConversionKHR;
#else
         dev->vk.DestroySamplerYcbcrConversionKHR = (PFN_vkDestroySamplerYcbcrConversionKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_KHR_imageless_framebuffer) {
   }
   if (dev->info.have_EXT_robustness2) {
   }
   if (dev->info.have_EXT_image_drm_format_modifier) {
#ifdef _WIN32
      if (!dev->vk.GetImageDrmFormatModifierPropertiesEXT) {
#ifndef NDEBUG
         dev->vk.GetImageDrmFormatModifierPropertiesEXT = (PFN_vkGetImageDrmFormatModifierPropertiesEXT)vk_stub_GetImageDrmFormatModifierPropertiesEXT;
#else
         dev->vk.GetImageDrmFormatModifierPropertiesEXT = (PFN_vkGetImageDrmFormatModifierPropertiesEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_vertex_attribute_divisor) {
   }
   if (dev->info.have_EXT_calibrated_timestamps) {
#ifdef _WIN32
      if (!dev->vk.GetCalibratedTimestampsEXT) {
#ifndef NDEBUG
         dev->vk.GetCalibratedTimestampsEXT = (PFN_vkGetCalibratedTimestampsEXT)vk_stub_GetCalibratedTimestampsEXT;
#else
         dev->vk.GetCalibratedTimestampsEXT = (PFN_vkGetCalibratedTimestampsEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_NV_linear_color_attachment) {
   }
   if (dev->info.have_KHR_dynamic_rendering) {
#ifdef _WIN32
      if (!dev->vk.CmdBeginRenderingKHR) {
#ifndef NDEBUG
         dev->vk.CmdBeginRenderingKHR = (PFN_vkCmdBeginRenderingKHR)vk_stub_CmdBeginRenderingKHR;
#else
         dev->vk.CmdBeginRenderingKHR = (PFN_vkCmdBeginRenderingKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdEndRenderingKHR) {
#ifndef NDEBUG
         dev->vk.CmdEndRenderingKHR = (PFN_vkCmdEndRenderingKHR)vk_stub_CmdEndRenderingKHR;
#else
         dev->vk.CmdEndRenderingKHR = (PFN_vkCmdEndRenderingKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_KHR_shader_clock) {
   }
   if (dev->info.have_EXT_sampler_filter_minmax) {
   }
   if (dev->info.have_EXT_custom_border_color) {
   }
   if (dev->info.have_EXT_non_seamless_cube_map) {
   }
   if (dev->info.have_EXT_border_color_swizzle) {
   }
   if (dev->info.have_EXT_blend_operation_advanced) {
   }
   if (dev->info.have_EXT_extended_dynamic_state) {
#ifdef _WIN32
      if (!dev->vk.CmdSetCullModeEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetCullModeEXT = (PFN_vkCmdSetCullModeEXT)vk_stub_CmdSetCullModeEXT;
#else
         dev->vk.CmdSetCullModeEXT = (PFN_vkCmdSetCullModeEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetFrontFaceEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetFrontFaceEXT = (PFN_vkCmdSetFrontFaceEXT)vk_stub_CmdSetFrontFaceEXT;
#else
         dev->vk.CmdSetFrontFaceEXT = (PFN_vkCmdSetFrontFaceEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetPrimitiveTopologyEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetPrimitiveTopologyEXT = (PFN_vkCmdSetPrimitiveTopologyEXT)vk_stub_CmdSetPrimitiveTopologyEXT;
#else
         dev->vk.CmdSetPrimitiveTopologyEXT = (PFN_vkCmdSetPrimitiveTopologyEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetViewportWithCountEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetViewportWithCountEXT = (PFN_vkCmdSetViewportWithCountEXT)vk_stub_CmdSetViewportWithCountEXT;
#else
         dev->vk.CmdSetViewportWithCountEXT = (PFN_vkCmdSetViewportWithCountEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetScissorWithCountEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetScissorWithCountEXT = (PFN_vkCmdSetScissorWithCountEXT)vk_stub_CmdSetScissorWithCountEXT;
#else
         dev->vk.CmdSetScissorWithCountEXT = (PFN_vkCmdSetScissorWithCountEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdBindVertexBuffers2EXT) {
#ifndef NDEBUG
         dev->vk.CmdBindVertexBuffers2EXT = (PFN_vkCmdBindVertexBuffers2EXT)vk_stub_CmdBindVertexBuffers2EXT;
#else
         dev->vk.CmdBindVertexBuffers2EXT = (PFN_vkCmdBindVertexBuffers2EXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetDepthTestEnableEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetDepthTestEnableEXT = (PFN_vkCmdSetDepthTestEnableEXT)vk_stub_CmdSetDepthTestEnableEXT;
#else
         dev->vk.CmdSetDepthTestEnableEXT = (PFN_vkCmdSetDepthTestEnableEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetDepthWriteEnableEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetDepthWriteEnableEXT = (PFN_vkCmdSetDepthWriteEnableEXT)vk_stub_CmdSetDepthWriteEnableEXT;
#else
         dev->vk.CmdSetDepthWriteEnableEXT = (PFN_vkCmdSetDepthWriteEnableEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetDepthCompareOpEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetDepthCompareOpEXT = (PFN_vkCmdSetDepthCompareOpEXT)vk_stub_CmdSetDepthCompareOpEXT;
#else
         dev->vk.CmdSetDepthCompareOpEXT = (PFN_vkCmdSetDepthCompareOpEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetDepthBoundsTestEnableEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetDepthBoundsTestEnableEXT = (PFN_vkCmdSetDepthBoundsTestEnableEXT)vk_stub_CmdSetDepthBoundsTestEnableEXT;
#else
         dev->vk.CmdSetDepthBoundsTestEnableEXT = (PFN_vkCmdSetDepthBoundsTestEnableEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetStencilTestEnableEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetStencilTestEnableEXT = (PFN_vkCmdSetStencilTestEnableEXT)vk_stub_CmdSetStencilTestEnableEXT;
#else
         dev->vk.CmdSetStencilTestEnableEXT = (PFN_vkCmdSetStencilTestEnableEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetStencilOpEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetStencilOpEXT = (PFN_vkCmdSetStencilOpEXT)vk_stub_CmdSetStencilOpEXT;
#else
         dev->vk.CmdSetStencilOpEXT = (PFN_vkCmdSetStencilOpEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_extended_dynamic_state2) {
#ifdef _WIN32
      if (!dev->vk.CmdSetPatchControlPointsEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetPatchControlPointsEXT = (PFN_vkCmdSetPatchControlPointsEXT)vk_stub_CmdSetPatchControlPointsEXT;
#else
         dev->vk.CmdSetPatchControlPointsEXT = (PFN_vkCmdSetPatchControlPointsEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetRasterizerDiscardEnableEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetRasterizerDiscardEnableEXT = (PFN_vkCmdSetRasterizerDiscardEnableEXT)vk_stub_CmdSetRasterizerDiscardEnableEXT;
#else
         dev->vk.CmdSetRasterizerDiscardEnableEXT = (PFN_vkCmdSetRasterizerDiscardEnableEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetDepthBiasEnableEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetDepthBiasEnableEXT = (PFN_vkCmdSetDepthBiasEnableEXT)vk_stub_CmdSetDepthBiasEnableEXT;
#else
         dev->vk.CmdSetDepthBiasEnableEXT = (PFN_vkCmdSetDepthBiasEnableEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetLogicOpEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetLogicOpEXT = (PFN_vkCmdSetLogicOpEXT)vk_stub_CmdSetLogicOpEXT;
#else
         dev->vk.CmdSetLogicOpEXT = (PFN_vkCmdSetLogicOpEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdSetPrimitiveRestartEnableEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetPrimitiveRestartEnableEXT = (PFN_vkCmdSetPrimitiveRestartEnableEXT)vk_stub_CmdSetPrimitiveRestartEnableEXT;
#else
         dev->vk.CmdSetPrimitiveRestartEnableEXT = (PFN_vkCmdSetPrimitiveRestartEnableEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_pipeline_creation_cache_control) {
   }
   if (dev->info.have_EXT_shader_stencil_export) {
   }
#ifdef VK_KHR_PORTABILITY_SUBSET_EXTENSION_NAME
   if (dev->info.have_KHR_portability_subset) {
   }
#endif
   if (dev->info.have_NV_compute_shader_derivatives) {
   }
   if (dev->info.have_KHR_timeline_semaphore) {
#ifdef _WIN32
      if (!dev->vk.GetSemaphoreCounterValueKHR) {
#ifndef NDEBUG
         dev->vk.GetSemaphoreCounterValueKHR = (PFN_vkGetSemaphoreCounterValueKHR)vk_stub_GetSemaphoreCounterValueKHR;
#else
         dev->vk.GetSemaphoreCounterValueKHR = (PFN_vkGetSemaphoreCounterValueKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.WaitSemaphoresKHR) {
#ifndef NDEBUG
         dev->vk.WaitSemaphoresKHR = (PFN_vkWaitSemaphoresKHR)vk_stub_WaitSemaphoresKHR;
#else
         dev->vk.WaitSemaphoresKHR = (PFN_vkWaitSemaphoresKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.SignalSemaphoreKHR) {
#ifndef NDEBUG
         dev->vk.SignalSemaphoreKHR = (PFN_vkSignalSemaphoreKHR)vk_stub_SignalSemaphoreKHR;
#else
         dev->vk.SignalSemaphoreKHR = (PFN_vkSignalSemaphoreKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_color_write_enable) {
#ifdef _WIN32
      if (!dev->vk.CmdSetColorWriteEnableEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetColorWriteEnableEXT = (PFN_vkCmdSetColorWriteEnableEXT)vk_stub_CmdSetColorWriteEnableEXT;
#else
         dev->vk.CmdSetColorWriteEnableEXT = (PFN_vkCmdSetColorWriteEnableEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_4444_formats) {
   }
   if (dev->info.have_EXT_scalar_block_layout) {
   }
   if (dev->info.have_KHR_swapchain) {
#ifdef _WIN32
      if (!dev->vk.CreateSwapchainKHR) {
#ifndef NDEBUG
         dev->vk.CreateSwapchainKHR = (PFN_vkCreateSwapchainKHR)vk_stub_CreateSwapchainKHR;
#else
         dev->vk.CreateSwapchainKHR = (PFN_vkCreateSwapchainKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.DestroySwapchainKHR) {
#ifndef NDEBUG
         dev->vk.DestroySwapchainKHR = (PFN_vkDestroySwapchainKHR)vk_stub_DestroySwapchainKHR;
#else
         dev->vk.DestroySwapchainKHR = (PFN_vkDestroySwapchainKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetSwapchainImagesKHR) {
#ifndef NDEBUG
         dev->vk.GetSwapchainImagesKHR = (PFN_vkGetSwapchainImagesKHR)vk_stub_GetSwapchainImagesKHR;
#else
         dev->vk.GetSwapchainImagesKHR = (PFN_vkGetSwapchainImagesKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.AcquireNextImageKHR) {
#ifndef NDEBUG
         dev->vk.AcquireNextImageKHR = (PFN_vkAcquireNextImageKHR)vk_stub_AcquireNextImageKHR;
#else
         dev->vk.AcquireNextImageKHR = (PFN_vkAcquireNextImageKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.QueuePresentKHR) {
#ifndef NDEBUG
         dev->vk.QueuePresentKHR = (PFN_vkQueuePresentKHR)vk_stub_QueuePresentKHR;
#else
         dev->vk.QueuePresentKHR = (PFN_vkQueuePresentKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetDeviceGroupPresentCapabilitiesKHR) {
#ifndef NDEBUG
         dev->vk.GetDeviceGroupPresentCapabilitiesKHR = (PFN_vkGetDeviceGroupPresentCapabilitiesKHR)vk_stub_GetDeviceGroupPresentCapabilitiesKHR;
#else
         dev->vk.GetDeviceGroupPresentCapabilitiesKHR = (PFN_vkGetDeviceGroupPresentCapabilitiesKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetDeviceGroupSurfacePresentModesKHR) {
#ifndef NDEBUG
         dev->vk.GetDeviceGroupSurfacePresentModesKHR = (PFN_vkGetDeviceGroupSurfacePresentModesKHR)vk_stub_GetDeviceGroupSurfacePresentModesKHR;
#else
         dev->vk.GetDeviceGroupSurfacePresentModesKHR = (PFN_vkGetDeviceGroupSurfacePresentModesKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.AcquireNextImage2KHR) {
#ifndef NDEBUG
         dev->vk.AcquireNextImage2KHR = (PFN_vkAcquireNextImage2KHR)vk_stub_AcquireNextImage2KHR;
#else
         dev->vk.AcquireNextImage2KHR = (PFN_vkAcquireNextImage2KHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_rasterization_order_attachment_access) {
   }
   if (dev->info.have_KHR_shader_float16_int8) {
   }
   if (dev->info.have_EXT_multi_draw) {
#ifdef _WIN32
      if (!dev->vk.CmdDrawMultiEXT) {
#ifndef NDEBUG
         dev->vk.CmdDrawMultiEXT = (PFN_vkCmdDrawMultiEXT)vk_stub_CmdDrawMultiEXT;
#else
         dev->vk.CmdDrawMultiEXT = (PFN_vkCmdDrawMultiEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdDrawMultiIndexedEXT) {
#ifndef NDEBUG
         dev->vk.CmdDrawMultiIndexedEXT = (PFN_vkCmdDrawMultiIndexedEXT)vk_stub_CmdDrawMultiIndexedEXT;
#else
         dev->vk.CmdDrawMultiIndexedEXT = (PFN_vkCmdDrawMultiIndexedEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_primitives_generated_query) {
   }
   if (dev->info.have_KHR_pipeline_library) {
   }
   if (dev->info.have_EXT_graphics_pipeline_library) {
   }
   if (dev->info.have_KHR_push_descriptor) {
#ifdef _WIN32
      if (!dev->vk.CmdPushDescriptorSetKHR) {
#ifndef NDEBUG
         dev->vk.CmdPushDescriptorSetKHR = (PFN_vkCmdPushDescriptorSetKHR)vk_stub_CmdPushDescriptorSetKHR;
#else
         dev->vk.CmdPushDescriptorSetKHR = (PFN_vkCmdPushDescriptorSetKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdPushDescriptorSetWithTemplateKHR) {
#ifndef NDEBUG
         dev->vk.CmdPushDescriptorSetWithTemplateKHR = (PFN_vkCmdPushDescriptorSetWithTemplateKHR)vk_stub_CmdPushDescriptorSetWithTemplateKHR;
#else
         dev->vk.CmdPushDescriptorSetWithTemplateKHR = (PFN_vkCmdPushDescriptorSetWithTemplateKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdPushDescriptorSetWithTemplateKHR) {
#ifndef NDEBUG
         dev->vk.CmdPushDescriptorSetWithTemplateKHR = (PFN_vkCmdPushDescriptorSetWithTemplateKHR)vk_stub_CmdPushDescriptorSetWithTemplateKHR;
#else
         dev->vk.CmdPushDescriptorSetWithTemplateKHR = (PFN_vkCmdPushDescriptorSetWithTemplateKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_KHR_descriptor_update_template) {
#ifdef _WIN32
      if (!dev->vk.CreateDescriptorUpdateTemplateKHR) {
#ifndef NDEBUG
         dev->vk.CreateDescriptorUpdateTemplateKHR = (PFN_vkCreateDescriptorUpdateTemplateKHR)vk_stub_CreateDescriptorUpdateTemplateKHR;
#else
         dev->vk.CreateDescriptorUpdateTemplateKHR = (PFN_vkCreateDescriptorUpdateTemplateKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.DestroyDescriptorUpdateTemplateKHR) {
#ifndef NDEBUG
         dev->vk.DestroyDescriptorUpdateTemplateKHR = (PFN_vkDestroyDescriptorUpdateTemplateKHR)vk_stub_DestroyDescriptorUpdateTemplateKHR;
#else
         dev->vk.DestroyDescriptorUpdateTemplateKHR = (PFN_vkDestroyDescriptorUpdateTemplateKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.UpdateDescriptorSetWithTemplateKHR) {
#ifndef NDEBUG
         dev->vk.UpdateDescriptorSetWithTemplateKHR = (PFN_vkUpdateDescriptorSetWithTemplateKHR)vk_stub_UpdateDescriptorSetWithTemplateKHR;
#else
         dev->vk.UpdateDescriptorSetWithTemplateKHR = (PFN_vkUpdateDescriptorSetWithTemplateKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CmdPushDescriptorSetWithTemplateKHR) {
#ifndef NDEBUG
         dev->vk.CmdPushDescriptorSetWithTemplateKHR = (PFN_vkCmdPushDescriptorSetWithTemplateKHR)vk_stub_CmdPushDescriptorSetWithTemplateKHR;
#else
         dev->vk.CmdPushDescriptorSetWithTemplateKHR = (PFN_vkCmdPushDescriptorSetWithTemplateKHR)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_line_rasterization) {
#ifdef _WIN32
      if (!dev->vk.CmdSetLineStippleEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetLineStippleEXT = (PFN_vkCmdSetLineStippleEXT)vk_stub_CmdSetLineStippleEXT;
#else
         dev->vk.CmdSetLineStippleEXT = (PFN_vkCmdSetLineStippleEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_mutable_descriptor_type) {
   }
   if (dev->info.have_EXT_vertex_input_dynamic_state) {
#ifdef _WIN32
      if (!dev->vk.CmdSetVertexInputEXT) {
#ifndef NDEBUG
         dev->vk.CmdSetVertexInputEXT = (PFN_vkCmdSetVertexInputEXT)vk_stub_CmdSetVertexInputEXT;
#else
         dev->vk.CmdSetVertexInputEXT = (PFN_vkCmdSetVertexInputEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
   if (dev->info.have_EXT_primitive_topology_list_restart) {
   }
   if (dev->info.have_KHR_dedicated_allocation) {
   }
   if (dev->info.have_EXT_descriptor_indexing) {
   }
   if (dev->info.have_EXT_depth_clip_enable) {
   }
   if (dev->info.have_EXT_shader_demote_to_helper_invocation) {
   }
   if (dev->info.have_KHR_shader_terminate_invocation) {
   }
   if (dev->info.have_EXT_host_image_copy) {
#ifdef _WIN32
      if (!dev->vk.CopyMemoryToImageEXT) {
#ifndef NDEBUG
         dev->vk.CopyMemoryToImageEXT = (PFN_vkCopyMemoryToImageEXT)vk_stub_CopyMemoryToImageEXT;
#else
         dev->vk.CopyMemoryToImageEXT = (PFN_vkCopyMemoryToImageEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CopyImageToMemoryEXT) {
#ifndef NDEBUG
         dev->vk.CopyImageToMemoryEXT = (PFN_vkCopyImageToMemoryEXT)vk_stub_CopyImageToMemoryEXT;
#else
         dev->vk.CopyImageToMemoryEXT = (PFN_vkCopyImageToMemoryEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.CopyImageToImageEXT) {
#ifndef NDEBUG
         dev->vk.CopyImageToImageEXT = (PFN_vkCopyImageToImageEXT)vk_stub_CopyImageToImageEXT;
#else
         dev->vk.CopyImageToImageEXT = (PFN_vkCopyImageToImageEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.TransitionImageLayoutEXT) {
#ifndef NDEBUG
         dev->vk.TransitionImageLayoutEXT = (PFN_vkTransitionImageLayoutEXT)vk_stub_TransitionImageLayoutEXT;
#else
         dev->vk.TransitionImageLayoutEXT = (PFN_vkTransitionImageLayoutEXT)vk_stub_function_not_loaded;
#endif
      }
#endif
#ifdef _WIN32
      if (!dev->vk.GetImageSubresourceLayout2EXT) {
#ifndef NDEBUG
         dev->vk.GetImageSubresourceLayout2EXT = (PFN_vkGetImageSubresourceLayout2EXT)vk_stub_GetImageSubresourceLayout2EXT;
#else
         dev->vk.GetImageSubresourceLayout2EXT = (PFN_vkGetImageSubresourceLayout2EXT)vk_stub_function_not_loaded;
#endif
      }
#endif
   }
}

#ifndef NDEBUG
/* generated stub functions */


   
void
vk_stub_TrimCommandPoolKHR()
{
   fprintf(stderr, "VK: vkTrimCommandPoolKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetDescriptorSetLayoutSupportKHR()
{
   fprintf(stderr, "VK: vkGetDescriptorSetLayoutSupportKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetDeviceBufferMemoryRequirementsKHR()
{
   fprintf(stderr, "VK: vkGetDeviceBufferMemoryRequirementsKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetDeviceImageMemoryRequirementsKHR()
{
   fprintf(stderr, "VK: vkGetDeviceImageMemoryRequirementsKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetDeviceImageSparseMemoryRequirementsKHR()
{
   fprintf(stderr, "VK: vkGetDeviceImageSparseMemoryRequirementsKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetMemoryFdKHR()
{
   fprintf(stderr, "VK: vkGetMemoryFdKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetMemoryFdPropertiesKHR()
{
   fprintf(stderr, "VK: vkGetMemoryFdPropertiesKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_ImportSemaphoreFdKHR()
{
   fprintf(stderr, "VK: vkImportSemaphoreFdKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetSemaphoreFdKHR()
{
   fprintf(stderr, "VK: vkGetSemaphoreFdKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CreateRenderPass2KHR()
{
   fprintf(stderr, "VK: vkCreateRenderPass2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdBeginRenderPass2KHR()
{
   fprintf(stderr, "VK: vkCmdBeginRenderPass2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdNextSubpass2KHR()
{
   fprintf(stderr, "VK: vkCmdNextSubpass2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdEndRenderPass2KHR()
{
   fprintf(stderr, "VK: vkCmdEndRenderPass2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetEvent2KHR()
{
   fprintf(stderr, "VK: vkCmdSetEvent2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdResetEvent2KHR()
{
   fprintf(stderr, "VK: vkCmdResetEvent2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdWaitEvents2KHR()
{
   fprintf(stderr, "VK: vkCmdWaitEvents2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdPipelineBarrier2KHR()
{
   fprintf(stderr, "VK: vkCmdPipelineBarrier2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdWriteTimestamp2KHR()
{
   fprintf(stderr, "VK: vkCmdWriteTimestamp2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_QueueSubmit2KHR()
{
   fprintf(stderr, "VK: vkQueueSubmit2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdWriteBufferMarker2AMD()
{
   fprintf(stderr, "VK: vkCmdWriteBufferMarker2AMD is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetQueueCheckpointData2NV()
{
   fprintf(stderr, "VK: vkGetQueueCheckpointData2NV is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetImageMemoryRequirements2KHR()
{
   fprintf(stderr, "VK: vkGetImageMemoryRequirements2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetBufferMemoryRequirements2KHR()
{
   fprintf(stderr, "VK: vkGetBufferMemoryRequirements2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetImageSparseMemoryRequirements2KHR()
{
   fprintf(stderr, "VK: vkGetImageSparseMemoryRequirements2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdDrawIndirectCountKHR()
{
   fprintf(stderr, "VK: vkCmdDrawIndirectCountKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdDrawIndexedIndirectCountKHR()
{
   fprintf(stderr, "VK: vkCmdDrawIndexedIndirectCountKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetSampleLocationsEXT()
{
   fprintf(stderr, "VK: vkCmdSetSampleLocationsEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetBufferDeviceAddressKHR()
{
   fprintf(stderr, "VK: vkGetBufferDeviceAddressKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetBufferOpaqueCaptureAddressKHR()
{
   fprintf(stderr, "VK: vkGetBufferOpaqueCaptureAddressKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetDeviceMemoryOpaqueCaptureAddressKHR()
{
   fprintf(stderr, "VK: vkGetDeviceMemoryOpaqueCaptureAddressKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetDescriptorSetLayoutSizeEXT()
{
   fprintf(stderr, "VK: vkGetDescriptorSetLayoutSizeEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetDescriptorSetLayoutBindingOffsetEXT()
{
   fprintf(stderr, "VK: vkGetDescriptorSetLayoutBindingOffsetEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetDescriptorEXT()
{
   fprintf(stderr, "VK: vkGetDescriptorEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdBindDescriptorBuffersEXT()
{
   fprintf(stderr, "VK: vkCmdBindDescriptorBuffersEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetDescriptorBufferOffsetsEXT()
{
   fprintf(stderr, "VK: vkCmdSetDescriptorBufferOffsetsEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdBindDescriptorBufferEmbeddedSamplersEXT()
{
   fprintf(stderr, "VK: vkCmdBindDescriptorBufferEmbeddedSamplersEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetBufferOpaqueCaptureDescriptorDataEXT()
{
   fprintf(stderr, "VK: vkGetBufferOpaqueCaptureDescriptorDataEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetImageOpaqueCaptureDescriptorDataEXT()
{
   fprintf(stderr, "VK: vkGetImageOpaqueCaptureDescriptorDataEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetImageViewOpaqueCaptureDescriptorDataEXT()
{
   fprintf(stderr, "VK: vkGetImageViewOpaqueCaptureDescriptorDataEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetSamplerOpaqueCaptureDescriptorDataEXT()
{
   fprintf(stderr, "VK: vkGetSamplerOpaqueCaptureDescriptorDataEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetAccelerationStructureOpaqueCaptureDescriptorDataEXT()
{
   fprintf(stderr, "VK: vkGetAccelerationStructureOpaqueCaptureDescriptorDataEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdBeginConditionalRenderingEXT()
{
   fprintf(stderr, "VK: vkCmdBeginConditionalRenderingEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdEndConditionalRenderingEXT()
{
   fprintf(stderr, "VK: vkCmdEndConditionalRenderingEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdBindTransformFeedbackBuffersEXT()
{
   fprintf(stderr, "VK: vkCmdBindTransformFeedbackBuffersEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdBeginTransformFeedbackEXT()
{
   fprintf(stderr, "VK: vkCmdBeginTransformFeedbackEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdEndTransformFeedbackEXT()
{
   fprintf(stderr, "VK: vkCmdEndTransformFeedbackEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdBeginQueryIndexedEXT()
{
   fprintf(stderr, "VK: vkCmdBeginQueryIndexedEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdEndQueryIndexedEXT()
{
   fprintf(stderr, "VK: vkCmdEndQueryIndexedEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdDrawIndirectByteCountEXT()
{
   fprintf(stderr, "VK: vkCmdDrawIndirectByteCountEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CreateSamplerYcbcrConversionKHR()
{
   fprintf(stderr, "VK: vkCreateSamplerYcbcrConversionKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_DestroySamplerYcbcrConversionKHR()
{
   fprintf(stderr, "VK: vkDestroySamplerYcbcrConversionKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetImageDrmFormatModifierPropertiesEXT()
{
   fprintf(stderr, "VK: vkGetImageDrmFormatModifierPropertiesEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetCalibratedTimestampsEXT()
{
   fprintf(stderr, "VK: vkGetCalibratedTimestampsEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdBeginRenderingKHR()
{
   fprintf(stderr, "VK: vkCmdBeginRenderingKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdEndRenderingKHR()
{
   fprintf(stderr, "VK: vkCmdEndRenderingKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetCullModeEXT()
{
   fprintf(stderr, "VK: vkCmdSetCullModeEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetFrontFaceEXT()
{
   fprintf(stderr, "VK: vkCmdSetFrontFaceEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetPrimitiveTopologyEXT()
{
   fprintf(stderr, "VK: vkCmdSetPrimitiveTopologyEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetViewportWithCountEXT()
{
   fprintf(stderr, "VK: vkCmdSetViewportWithCountEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetScissorWithCountEXT()
{
   fprintf(stderr, "VK: vkCmdSetScissorWithCountEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdBindVertexBuffers2EXT()
{
   fprintf(stderr, "VK: vkCmdBindVertexBuffers2EXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetDepthTestEnableEXT()
{
   fprintf(stderr, "VK: vkCmdSetDepthTestEnableEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetDepthWriteEnableEXT()
{
   fprintf(stderr, "VK: vkCmdSetDepthWriteEnableEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetDepthCompareOpEXT()
{
   fprintf(stderr, "VK: vkCmdSetDepthCompareOpEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetDepthBoundsTestEnableEXT()
{
   fprintf(stderr, "VK: vkCmdSetDepthBoundsTestEnableEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetStencilTestEnableEXT()
{
   fprintf(stderr, "VK: vkCmdSetStencilTestEnableEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetStencilOpEXT()
{
   fprintf(stderr, "VK: vkCmdSetStencilOpEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetPatchControlPointsEXT()
{
   fprintf(stderr, "VK: vkCmdSetPatchControlPointsEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetRasterizerDiscardEnableEXT()
{
   fprintf(stderr, "VK: vkCmdSetRasterizerDiscardEnableEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetDepthBiasEnableEXT()
{
   fprintf(stderr, "VK: vkCmdSetDepthBiasEnableEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetLogicOpEXT()
{
   fprintf(stderr, "VK: vkCmdSetLogicOpEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetPrimitiveRestartEnableEXT()
{
   fprintf(stderr, "VK: vkCmdSetPrimitiveRestartEnableEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetSemaphoreCounterValueKHR()
{
   fprintf(stderr, "VK: vkGetSemaphoreCounterValueKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_WaitSemaphoresKHR()
{
   fprintf(stderr, "VK: vkWaitSemaphoresKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_SignalSemaphoreKHR()
{
   fprintf(stderr, "VK: vkSignalSemaphoreKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetColorWriteEnableEXT()
{
   fprintf(stderr, "VK: vkCmdSetColorWriteEnableEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CreateSwapchainKHR()
{
   fprintf(stderr, "VK: vkCreateSwapchainKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_DestroySwapchainKHR()
{
   fprintf(stderr, "VK: vkDestroySwapchainKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetSwapchainImagesKHR()
{
   fprintf(stderr, "VK: vkGetSwapchainImagesKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_AcquireNextImageKHR()
{
   fprintf(stderr, "VK: vkAcquireNextImageKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_QueuePresentKHR()
{
   fprintf(stderr, "VK: vkQueuePresentKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetDeviceGroupPresentCapabilitiesKHR()
{
   fprintf(stderr, "VK: vkGetDeviceGroupPresentCapabilitiesKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetDeviceGroupSurfacePresentModesKHR()
{
   fprintf(stderr, "VK: vkGetDeviceGroupSurfacePresentModesKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_AcquireNextImage2KHR()
{
   fprintf(stderr, "VK: vkAcquireNextImage2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdDrawMultiEXT()
{
   fprintf(stderr, "VK: vkCmdDrawMultiEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdDrawMultiIndexedEXT()
{
   fprintf(stderr, "VK: vkCmdDrawMultiIndexedEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdPushDescriptorSetKHR()
{
   fprintf(stderr, "VK: vkCmdPushDescriptorSetKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdPushDescriptorSetWithTemplateKHR()
{
   fprintf(stderr, "VK: vkCmdPushDescriptorSetWithTemplateKHR is not loaded properly!\n");
   abort();
}
      
void
vk_stub_CreateDescriptorUpdateTemplateKHR()
{
   fprintf(stderr, "VK: vkCreateDescriptorUpdateTemplateKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_DestroyDescriptorUpdateTemplateKHR()
{
   fprintf(stderr, "VK: vkDestroyDescriptorUpdateTemplateKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_UpdateDescriptorSetWithTemplateKHR()
{
   fprintf(stderr, "VK: vkUpdateDescriptorSetWithTemplateKHR is not loaded properly!\n");
   abort();
}
      
void
vk_stub_CmdSetLineStippleEXT()
{
   fprintf(stderr, "VK: vkCmdSetLineStippleEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CmdSetVertexInputEXT()
{
   fprintf(stderr, "VK: vkCmdSetVertexInputEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CopyMemoryToImageEXT()
{
   fprintf(stderr, "VK: vkCopyMemoryToImageEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CopyImageToMemoryEXT()
{
   fprintf(stderr, "VK: vkCopyImageToMemoryEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CopyImageToImageEXT()
{
   fprintf(stderr, "VK: vkCopyImageToImageEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_TransitionImageLayoutEXT()
{
   fprintf(stderr, "VK: vkTransitionImageLayoutEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetImageSubresourceLayout2EXT()
{
   fprintf(stderr, "VK: vkGetImageSubresourceLayout2EXT is not loaded properly!\n");
   abort();
}
#endif
