#ifndef VK_DEVICE_INFO_H
#define VK_DEVICE_INFO_H

#include <stdbool.h>
#include <vulkan/vulkan.h>

struct vk_device;

struct vk_device_info {
   uint32_t device_version;

   bool have_KHR_maintenance1;
   bool have_KHR_maintenance2;
   bool have_KHR_maintenance3;
   bool have_KHR_maintenance4;
   bool have_KHR_external_memory;
   bool have_KHR_external_memory_fd;
   bool have_KHR_vulkan_memory_model;
   bool have_KHR_external_semaphore_fd;
   bool have_KHR_create_renderpass2;
   bool have_KHR_synchronization2;
   bool have_KHR_swapchain_mutable_format;
   bool have_EXT_provoking_vertex;
   bool have_EXT_shader_viewport_index_layer;
   bool have_KHR_get_memory_requirements2;
   bool have_EXT_post_depth_coverage;
   bool have_EXT_depth_clip_control;
   bool have_EXT_shader_subgroup_ballot;
   bool have_EXT_shader_subgroup_vote;
   bool have_EXT_shader_atomic_float;
   bool have_KHR_shader_atomic_int64;
   bool have_KHR_8bit_storage;
   bool have_KHR_16bit_storage;
   bool have_EXT_image_2d_view_of_3d;
   bool have_KHR_driver_properties;
   bool have_EXT_memory_budget;
   bool have_KHR_draw_indirect_count;
   bool have_EXT_fragment_shader_interlock;
   bool have_EXT_sample_locations;
   bool have_EXT_conservative_rasterization;
   bool have_KHR_shader_draw_parameters;
   bool have_KHR_sampler_mirror_clamp_to_edge;
   bool have_KHR_buffer_device_address;
   bool have_EXT_descriptor_buffer;
   bool have_EXT_conditional_rendering;
   bool have_EXT_transform_feedback;
   bool have_EXT_index_type_uint8;
   bool have_KHR_image_format_list;
   bool have_KHR_sampler_ycbcr_conversion;
   bool have_KHR_imageless_framebuffer;
   bool have_EXT_robustness2;
   bool have_EXT_image_drm_format_modifier;
   bool have_EXT_vertex_attribute_divisor;
   bool have_EXT_calibrated_timestamps;
   bool have_NV_linear_color_attachment;
   bool have_KHR_dynamic_rendering;
   bool have_KHR_shader_clock;
   bool have_EXT_sampler_filter_minmax;
   bool have_EXT_custom_border_color;
   bool have_EXT_non_seamless_cube_map;
   bool have_EXT_border_color_swizzle;
   bool have_EXT_blend_operation_advanced;
   bool have_EXT_extended_dynamic_state;
   bool have_EXT_extended_dynamic_state2;
   bool have_EXT_pipeline_creation_cache_control;
   bool have_EXT_shader_stencil_export;
#ifdef VK_KHR_PORTABILITY_SUBSET_EXTENSION_NAME
   bool have_KHR_portability_subset;
#endif
   bool have_NV_compute_shader_derivatives;
   bool have_KHR_timeline_semaphore;
   bool have_EXT_color_write_enable;
   bool have_EXT_4444_formats;
   bool have_EXT_scalar_block_layout;
   bool have_KHR_swapchain;
   bool have_EXT_rasterization_order_attachment_access;
   bool have_KHR_shader_float16_int8;
   bool have_EXT_multi_draw;
   bool have_EXT_primitives_generated_query;
   bool have_KHR_pipeline_library;
   bool have_EXT_graphics_pipeline_library;
   bool have_KHR_push_descriptor;
   bool have_KHR_descriptor_update_template;
   bool have_EXT_line_rasterization;
   bool have_EXT_mutable_descriptor_type;
   bool have_EXT_vertex_input_dynamic_state;
   bool have_EXT_primitive_topology_list_restart;
   bool have_KHR_dedicated_allocation;
   bool have_EXT_descriptor_indexing;
   bool have_EXT_depth_clip_enable;
   bool have_EXT_shader_demote_to_helper_invocation;
   bool have_KHR_shader_terminate_invocation;
   bool have_EXT_host_image_copy;
   bool have_vulkan11;
   bool have_vulkan12;

   VkPhysicalDeviceFeatures2 feats;
   VkPhysicalDeviceSubgroupProperties subgroup;
   VkPhysicalDeviceVulkan11Features feats11;
   VkPhysicalDeviceVulkan12Features feats12;

   VkPhysicalDeviceProperties props;
   VkPhysicalDeviceVulkan11Properties props11;
   VkPhysicalDeviceVulkan12Properties props12;

   VkPhysicalDeviceMemoryProperties mem_props;
   VkPhysicalDeviceIDProperties deviceid_props;

   
   
   
   VkPhysicalDeviceMaintenance4FeaturesKHR maint4_feats;
   
   
   
   
   
   VkPhysicalDeviceSynchronization2FeaturesKHR sync2_feats;
   
   VkPhysicalDeviceProvokingVertexFeaturesEXT pv_feats;
   VkPhysicalDeviceProvokingVertexPropertiesEXT pv_props;
   
   
   
   VkPhysicalDeviceDepthClipControlFeaturesEXT clip_control_feats;
   
   
   VkPhysicalDeviceShaderAtomicFloatFeaturesEXT atomic_float_feats;
   VkPhysicalDeviceShaderAtomicInt64FeaturesKHR atomic_int_feats;
   VkPhysicalDevice8BitStorageFeaturesKHR storage_8bit_feats;
   VkPhysicalDevice16BitStorageFeaturesKHR storage_16bit_feats;
   VkPhysicalDeviceImage2DViewOf3DFeaturesEXT view2d_feats;
   VkPhysicalDeviceDriverPropertiesKHR driver_props;
   
   
   VkPhysicalDeviceFragmentShaderInterlockFeaturesEXT interlock_feats;
   VkPhysicalDeviceSampleLocationsPropertiesEXT sample_locations_props;
   VkPhysicalDeviceConservativeRasterizationPropertiesEXT cons_raster_props;
   
   
   VkPhysicalDeviceBufferDeviceAddressFeaturesKHR bda_feats;
   VkPhysicalDeviceDescriptorBufferFeaturesEXT db_feats;
   VkPhysicalDeviceDescriptorBufferPropertiesEXT db_props;
   VkPhysicalDeviceConditionalRenderingFeaturesEXT cond_render_feats;
   VkPhysicalDeviceTransformFeedbackFeaturesEXT tf_feats;
   VkPhysicalDeviceTransformFeedbackPropertiesEXT tf_props;
   VkPhysicalDeviceIndexTypeUint8FeaturesEXT index_uint8_feats;
   
   
   VkPhysicalDeviceImagelessFramebufferFeaturesKHR imgless_feats;
   VkPhysicalDeviceRobustness2FeaturesEXT rb2_feats;
   VkPhysicalDeviceRobustness2PropertiesEXT rb2_props;
   
   VkPhysicalDeviceVertexAttributeDivisorFeaturesEXT vdiv_feats;
   VkPhysicalDeviceVertexAttributeDivisorPropertiesEXT vdiv_props;
   
   VkPhysicalDeviceLinearColorAttachmentFeaturesNV linear_color_feats;
   VkPhysicalDeviceDynamicRenderingFeaturesKHR dynamic_render_feats;
   VkPhysicalDeviceShaderClockFeaturesKHR shader_clock_feats;
   VkPhysicalDeviceSamplerFilterMinmaxPropertiesEXT reduction_props;
   VkPhysicalDeviceCustomBorderColorFeaturesEXT border_color_feats;
   VkPhysicalDeviceCustomBorderColorPropertiesEXT border_color_props;
   VkPhysicalDeviceNonSeamlessCubeMapFeaturesEXT nonseamless_feats;
   VkPhysicalDeviceBorderColorSwizzleFeaturesEXT border_swizzle_feats;
   VkPhysicalDeviceBlendOperationAdvancedPropertiesEXT blend_props;
   VkPhysicalDeviceExtendedDynamicStateFeaturesEXT dynamic_state_feats;
   VkPhysicalDeviceExtendedDynamicState2FeaturesEXT dynamic_state2_feats;
   VkPhysicalDevicePipelineCreationCacheControlFeaturesEXT pipeline_cache_control_feats;
   
#ifdef VK_KHR_PORTABILITY_SUBSET_EXTENSION_NAME
   VkPhysicalDevicePortabilitySubsetFeaturesKHR portability_subset_feats;
#endif
   VkPhysicalDeviceComputeShaderDerivativesFeaturesNV shader_derivs_feats;
   VkPhysicalDeviceTimelineSemaphoreFeaturesKHR timeline_feats;
   VkPhysicalDeviceColorWriteEnableFeaturesEXT cwrite_feats;
   VkPhysicalDevice4444FormatsFeaturesEXT format_4444_feats;
   VkPhysicalDeviceScalarBlockLayoutFeaturesEXT scalar_block_layout_feats;
   
   VkPhysicalDeviceRasterizationOrderAttachmentAccessFeaturesEXT rast_order_access_feats;
   VkPhysicalDeviceShaderFloat16Int8FeaturesKHR shader_float16_int8_feats;
   VkPhysicalDeviceMultiDrawFeaturesEXT multidraw_feats;
   VkPhysicalDeviceMultiDrawPropertiesEXT multidraw_props;
   VkPhysicalDevicePrimitivesGeneratedQueryFeaturesEXT primgen_feats;
   
   VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT gpl_feats;
   VkPhysicalDeviceGraphicsPipelineLibraryPropertiesEXT gpl_props;
   VkPhysicalDevicePushDescriptorPropertiesKHR push_props;
   
   VkPhysicalDeviceLineRasterizationFeaturesEXT line_rast_feats;
   VkPhysicalDeviceLineRasterizationPropertiesEXT line_rast_props;
   VkPhysicalDeviceMutableDescriptorTypeFeaturesEXT mutable_feats;
   VkPhysicalDeviceVertexInputDynamicStateFeaturesEXT vertex_input_feats;
   VkPhysicalDevicePrimitiveTopologyListRestartFeaturesEXT list_restart_feats;
   
   VkPhysicalDeviceDescriptorIndexingFeaturesEXT desc_indexing_feats;
   VkPhysicalDeviceDescriptorIndexingPropertiesEXT desc_indexing_props;
   VkPhysicalDeviceDepthClipEnableFeaturesEXT depth_clip_enable_feats;
   VkPhysicalDeviceShaderDemoteToHelperInvocationFeaturesEXT demote_feats;
   VkPhysicalDeviceShaderTerminateInvocationFeaturesKHR terminate_feats;
   

    const char *extensions[80];
    uint32_t num_extensions;
};

bool
vk_get_physical_device_info(struct vk_device *dev);

void
vk_verify_device_extensions(struct vk_device *dev);

/* stub functions that get inserted into the dispatch table if they are not
 * properly loaded.
 */
void vk_stub_TrimCommandPoolKHR(void);
void vk_stub_GetDescriptorSetLayoutSupportKHR(void);
void vk_stub_GetDeviceBufferMemoryRequirementsKHR(void);
void vk_stub_GetDeviceImageMemoryRequirementsKHR(void);
void vk_stub_GetDeviceImageSparseMemoryRequirementsKHR(void);
void vk_stub_GetMemoryFdKHR(void);
void vk_stub_GetMemoryFdPropertiesKHR(void);
void vk_stub_ImportSemaphoreFdKHR(void);
void vk_stub_GetSemaphoreFdKHR(void);
void vk_stub_CreateRenderPass2KHR(void);
void vk_stub_CmdBeginRenderPass2KHR(void);
void vk_stub_CmdNextSubpass2KHR(void);
void vk_stub_CmdEndRenderPass2KHR(void);
void vk_stub_CmdSetEvent2KHR(void);
void vk_stub_CmdResetEvent2KHR(void);
void vk_stub_CmdWaitEvents2KHR(void);
void vk_stub_CmdPipelineBarrier2KHR(void);
void vk_stub_CmdWriteTimestamp2KHR(void);
void vk_stub_QueueSubmit2KHR(void);
void vk_stub_CmdWriteBufferMarker2AMD(void);
void vk_stub_GetQueueCheckpointData2NV(void);
void vk_stub_GetImageMemoryRequirements2KHR(void);
void vk_stub_GetBufferMemoryRequirements2KHR(void);
void vk_stub_GetImageSparseMemoryRequirements2KHR(void);
void vk_stub_CmdDrawIndirectCountKHR(void);
void vk_stub_CmdDrawIndexedIndirectCountKHR(void);
void vk_stub_CmdSetSampleLocationsEXT(void);
void vk_stub_GetBufferDeviceAddressKHR(void);
void vk_stub_GetBufferOpaqueCaptureAddressKHR(void);
void vk_stub_GetDeviceMemoryOpaqueCaptureAddressKHR(void);
void vk_stub_GetDescriptorSetLayoutSizeEXT(void);
void vk_stub_GetDescriptorSetLayoutBindingOffsetEXT(void);
void vk_stub_GetDescriptorEXT(void);
void vk_stub_CmdBindDescriptorBuffersEXT(void);
void vk_stub_CmdSetDescriptorBufferOffsetsEXT(void);
void vk_stub_CmdBindDescriptorBufferEmbeddedSamplersEXT(void);
void vk_stub_GetBufferOpaqueCaptureDescriptorDataEXT(void);
void vk_stub_GetImageOpaqueCaptureDescriptorDataEXT(void);
void vk_stub_GetImageViewOpaqueCaptureDescriptorDataEXT(void);
void vk_stub_GetSamplerOpaqueCaptureDescriptorDataEXT(void);
void vk_stub_GetAccelerationStructureOpaqueCaptureDescriptorDataEXT(void);
void vk_stub_CmdBeginConditionalRenderingEXT(void);
void vk_stub_CmdEndConditionalRenderingEXT(void);
void vk_stub_CmdBindTransformFeedbackBuffersEXT(void);
void vk_stub_CmdBeginTransformFeedbackEXT(void);
void vk_stub_CmdEndTransformFeedbackEXT(void);
void vk_stub_CmdBeginQueryIndexedEXT(void);
void vk_stub_CmdEndQueryIndexedEXT(void);
void vk_stub_CmdDrawIndirectByteCountEXT(void);
void vk_stub_CreateSamplerYcbcrConversionKHR(void);
void vk_stub_DestroySamplerYcbcrConversionKHR(void);
void vk_stub_GetImageDrmFormatModifierPropertiesEXT(void);
void vk_stub_GetCalibratedTimestampsEXT(void);
void vk_stub_CmdBeginRenderingKHR(void);
void vk_stub_CmdEndRenderingKHR(void);
void vk_stub_CmdSetCullModeEXT(void);
void vk_stub_CmdSetFrontFaceEXT(void);
void vk_stub_CmdSetPrimitiveTopologyEXT(void);
void vk_stub_CmdSetViewportWithCountEXT(void);
void vk_stub_CmdSetScissorWithCountEXT(void);
void vk_stub_CmdBindVertexBuffers2EXT(void);
void vk_stub_CmdSetDepthTestEnableEXT(void);
void vk_stub_CmdSetDepthWriteEnableEXT(void);
void vk_stub_CmdSetDepthCompareOpEXT(void);
void vk_stub_CmdSetDepthBoundsTestEnableEXT(void);
void vk_stub_CmdSetStencilTestEnableEXT(void);
void vk_stub_CmdSetStencilOpEXT(void);
void vk_stub_CmdSetPatchControlPointsEXT(void);
void vk_stub_CmdSetRasterizerDiscardEnableEXT(void);
void vk_stub_CmdSetDepthBiasEnableEXT(void);
void vk_stub_CmdSetLogicOpEXT(void);
void vk_stub_CmdSetPrimitiveRestartEnableEXT(void);
void vk_stub_GetSemaphoreCounterValueKHR(void);
void vk_stub_WaitSemaphoresKHR(void);
void vk_stub_SignalSemaphoreKHR(void);
void vk_stub_CmdSetColorWriteEnableEXT(void);
void vk_stub_CreateSwapchainKHR(void);
void vk_stub_DestroySwapchainKHR(void);
void vk_stub_GetSwapchainImagesKHR(void);
void vk_stub_AcquireNextImageKHR(void);
void vk_stub_QueuePresentKHR(void);
void vk_stub_GetDeviceGroupPresentCapabilitiesKHR(void);
void vk_stub_GetDeviceGroupSurfacePresentModesKHR(void);
void vk_stub_AcquireNextImage2KHR(void);
void vk_stub_CmdDrawMultiEXT(void);
void vk_stub_CmdDrawMultiIndexedEXT(void);
void vk_stub_CmdPushDescriptorSetKHR(void);
void vk_stub_CmdPushDescriptorSetWithTemplateKHR(void);
void vk_stub_CmdPushDescriptorSetWithTemplateKHR(void);
void vk_stub_CreateDescriptorUpdateTemplateKHR(void);
void vk_stub_DestroyDescriptorUpdateTemplateKHR(void);
void vk_stub_UpdateDescriptorSetWithTemplateKHR(void);
void vk_stub_CmdPushDescriptorSetWithTemplateKHR(void);
void vk_stub_CmdSetLineStippleEXT(void);
void vk_stub_CmdSetVertexInputEXT(void);
void vk_stub_CopyMemoryToImageEXT(void);
void vk_stub_CopyImageToMemoryEXT(void);
void vk_stub_CopyImageToImageEXT(void);
void vk_stub_TransitionImageLayoutEXT(void);
void vk_stub_GetImageSubresourceLayout2EXT(void);

#endif
